@@ -204,7 +204,8 @@ def handle_authorizers(method, path, data, headers):
 # -----------------------
 
 def get_domain_from_path(path):
-    return re.match(PATH_REGEX_PATH_MAPPING, path).group(1)
+    matched = re.match(PATH_REGEX_PATH_MAPPING, path)
+    return matched.group(1) if matched else None
 
 
 def get_base_path_from_path(path):
@@ -480,7 +481,7 @@ def get_cors_response(headers):
     response = Response()
     response.status_code = 200
     response.headers['Access-Control-Allow-Origin'] = '*'
-    response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, DELETE'
+    response.headers['Access-Control-Allow-Methods'] = 'GET, POST, PUT, DELETE, PATCH'
     response.headers['Access-Control-Allow-Headers'] = '*'
     response._content = ''
     return response
