@@ -31,11 +31,12 @@ from localstack.services.awslambda.lambda_utils import (
 from localstack.utils.analytics import event_publisher
 from localstack.utils.http_utils import parse_chunked_data
 from localstack.utils.aws.aws_models import LambdaFunction, CodeSigningConfig
-from localstack.services.cloudformation.service_models import LAMBDA_POLICY_NAME_PATTERN
 
 # logger
 LOG = logging.getLogger(__name__)
 
+# name pattern of IAM policies associated with Lambda functions
+LAMBDA_POLICY_NAME_PATTERN = 'lambda_policy_%s'
 # constants
 APP_NAME = 'lambda_api'
 PATH_ROOT = '/2015-03-31'
@@ -295,7 +296,7 @@ def process_apigateway_invocation(func_arn, path, payload, stage, api_id, header
                                   stage_variables={}, request_context={}, event_context={}):
     try:
         resource_path = resource_path or path
-        event = construct_invocation_event(method, resource_path, headers, payload, query_string_params)
+        event = construct_invocation_event(method, path, headers, payload, query_string_params)
         path_params = dict(path_params)
         fix_proxy_path_params(path_params)
         event['pathParameters'] = path_params
@@ -1009,6 +1010,20 @@ def get_lambda_policy(function, qualifier=None):
     return (policy or [None])[0]
 
 
+def lookup_function(function, region, request_url):
+    result = {
+        'Configuration': function,
+        'Code': {
+            'Location': '%s/code' % request_url
+        },
+        'Tags': function['Tags']
+    }
+    lambda_details = region.lambdas.get(function['FunctionArn'])
+    if lambda_details.concurrency is not None:
+        result['Concurrency'] = lambda_details.concurrency
+    return jsonify(result)
+
+
 def not_found_error(ref=None, msg=None):
     if not msg:
         msg = 'The resource you requested does not exist.'
@@ -1103,18 +1118,10 @@ def get_function(function):
     region = LambdaRegion.get()
     funcs = do_list_functions()
     for func in funcs:
-        if func['FunctionName'] == function:
-            result = {
-                'Configuration': func,
-                'Code': {
-                    'Location': '%s/code' % request.url
-                },
-                'Tags': func['Tags']
-            }
-            lambda_details = region.lambdas.get(func['FunctionArn'])
-            if lambda_details.concurrency is not None:
-                result['Concurrency'] = lambda_details.concurrency
-            return jsonify(result)
+        if function == func['FunctionName']:
+            return lookup_function(func, region, request.url)
+        elif function in func['FunctionArn']:
+            return lookup_function(func, region, request.url)
     return not_found_error(func_arn(function))
 
 
@@ -1309,20 +1316,25 @@ def generate_policy(sid, action, arn, sourcearn, principal):
 
 @app.route('%s/functions/<function>/policy' % PATH_ROOT, methods=['POST'])
 def add_permission(function):
-    region = LambdaRegion.get()
+    arn = func_arn(function)
+    qualifier = request.args.get('Qualifier')
+    q_arn = func_qualifier(function, qualifier)
+    result = add_permission_policy_statement(function, arn, q_arn)
+    return result
 
+
+def add_permission_policy_statement(resource_name, resource_arn, resource_arn_qualified):
+    region = LambdaRegion.get()
     data = json.loads(to_str(request.data))
     iam_client = aws_stack.connect_to_service('iam')
     sid = data.get('StatementId')
     action = data.get('Action')
     principal = data.get('Principal')
     sourcearn = data.get('SourceArn')
-    qualifier = request.args.get('Qualifier')
-    arn = func_arn(function)
-    previous_policy = get_lambda_policy(function)
+    previous_policy = get_lambda_policy(resource_name)
 
-    if arn not in region.lambdas:
-        return not_found_error(func_arn(function))
+    if resource_arn not in region.lambdas:
+        return not_found_error(resource_arn)
 
     if not re.match(r'lambda:[*]|lambda:[a-zA-Z]+|[*]', action):
         return error_response('1 validation error detected: Value "%s" at "action" failed to satisfy '
@@ -1330,21 +1342,23 @@ def add_permission(function):
                               '(lambda:[*]|lambda:[a-zA-Z]+|[*])' % action,
                               400, error_type='ValidationException')
 
-    q_arn = func_qualifier(function, qualifier)
-    new_policy = generate_policy(sid, action, q_arn, sourcearn, principal)
+    new_policy = generate_policy(sid, action, resource_arn_qualified, sourcearn, principal)
 
     if previous_policy:
         statment_with_sid = next((statement for statement in previous_policy['Statement'] if statement['Sid'] == sid),
             None)
         if statment_with_sid:
-            return error_response('The statement id (%s) provided already exists. Please provide a new statement id,'
-                        ' or remove the existing statement.' % sid, 400, error_type='ResourceConflictException')
+            msg = ('The statement id (%s) provided already exists. Please provide a new statement id, '
+                   'or remove the existing statement.') % sid
+            return error_response(msg, 400, error_type='ResourceConflictException')
 
         new_policy['Statement'].extend(previous_policy['Statement'])
         iam_client.delete_policy(PolicyArn=previous_policy['PolicyArn'])
 
-    iam_client.create_policy(PolicyName=LAMBDA_POLICY_NAME_PATTERN % function,
-        PolicyDocument=json.dumps(new_policy), Description='Policy for Lambda function "%s"' % function)
+    policy_name = LAMBDA_POLICY_NAME_PATTERN % resource_name
+    LOG.debug('Creating IAM policy "%s" for Lambda resource %s' % (policy_name, resource_arn))
+    iam_client.create_policy(PolicyName=policy_name, PolicyDocument=json.dumps(new_policy),
+        Description='Policy for Lambda function "%s"' % resource_name)
 
     result = {'Statement': json.dumps(new_policy['Statement'][0])}
     return jsonify(result)
