@@ -20,7 +20,7 @@ from localstack.utils import bootstrap
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
     CaptureOutput, FuncThread, TMP_FILES, short_uid, save_file, rm_rf, in_docker, long_uid,
-    now, to_str, to_bytes, run, cp_r, json_safe, get_free_tcp_port)
+    now, to_str, to_bytes, run, cp_r, json_safe, get_free_tcp_port, rm_docker_container)
 from localstack.services.install import INSTALL_PATH_LOCALSTACK_FAT_JAR
 from localstack.utils.aws.dead_letter_queue import lambda_error_to_dead_letter_queue
 from localstack.utils.aws.dead_letter_queue import sqs_error_to_dead_letter_queue
@@ -55,6 +55,9 @@ DOCKER_MAIN_CONTAINER_IP = None
 # maps lambda arns to concurrency locks
 LAMBDA_CONCURRENCY_LOCK = {}
 
+# CWD folder of handler code in Lambda containers
+DOCKER_TASK_FOLDER = '/var/task'
+
 
 class InvocationException(Exception):
     def __init__(self, message, log_output, result=None):
@@ -92,7 +95,7 @@ def _store_logs(func_details, log_output, invocation_time=None, container_id=Non
 
 def get_main_endpoint_from_container():
     global DOCKER_MAIN_CONTAINER_IP
-    if DOCKER_MAIN_CONTAINER_IP is None:
+    if not config.HOSTNAME_FROM_LAMBDA and DOCKER_MAIN_CONTAINER_IP is None:
         DOCKER_MAIN_CONTAINER_IP = False
         try:
             if in_docker():
@@ -102,8 +105,8 @@ def get_main_endpoint_from_container():
             container_name = bootstrap.get_main_container_name()
             LOG.info('Unable to get IP address of main Docker container "%s": %s' %
                 (container_name, e))
-    # return main container IP, or fall back to Docker host (bridge IP, or host DNS address)
-    return DOCKER_MAIN_CONTAINER_IP or config.DOCKER_HOST_FROM_CONTAINER
+    # return (1) predefined endpoint host, or (2) main container IP, or (3) Docker host (e.g., bridge IP)
+    return config.HOSTNAME_FROM_LAMBDA or DOCKER_MAIN_CONTAINER_IP or config.DOCKER_HOST_FROM_CONTAINER
 
 
 class InvocationResult(object):
@@ -295,7 +298,7 @@ class LambdaExecutorContainers(LambdaExecutor):
 
         # custom command to execute in the container
         command = ''
-        events_file = ''
+        events_file_path = ''
 
         if config.LAMBDA_JAVA_OPTS and is_java_lambda(runtime):
             # if running a Java Lambda with our custom executor, set up classpath arguments
@@ -306,12 +309,13 @@ class LambdaExecutorContainers(LambdaExecutor):
             if not os.path.exists(target_file):
                 cp_r(LAMBDA_EXECUTOR_JAR, target_file)
             # TODO cleanup once we have custom Java Docker image
-            taskdir = '/var/task'
             events_file = '_lambda.events.%s.json' % short_uid()
-            save_file(os.path.join(lambda_cwd, events_file), event_body)
+            events_file_path = os.path.join(lambda_cwd, events_file)
+            save_file(events_file_path, event_body)
+            # construct Java command
             classpath = Util.get_java_classpath(target_file)
             command = ("bash -c 'cd %s; java %s -cp \"%s\" \"%s\" \"%s\" \"%s\"'" %
-                (taskdir, java_opts, classpath, LAMBDA_EXECUTOR_CLASS, handler, events_file))
+                (DOCKER_TASK_FOLDER, java_opts, classpath, LAMBDA_EXECUTOR_CLASS, handler, events_file))
 
         # accept any self-signed certificates for outgoing calls from the Lambda
         if is_nodejs_runtime(runtime):
@@ -320,12 +324,18 @@ class LambdaExecutorContainers(LambdaExecutor):
         # determine the command to be executed (implemented by subclasses)
         cmd = self.prepare_execution(func_details, environment, command)
 
+        # copy events file into container, if necessary
+        if events_file_path:
+            container_name = self.get_container_name(func_details.arn())
+            self.copy_into_container(events_file_path, container_name, DOCKER_TASK_FOLDER)
+
         # run Lambda executor and fetch invocation result
         LOG.info('Running lambda cmd: %s' % cmd)
         result = self.run_lambda_executor(cmd, stdin, env_vars=environment, func_details=func_details)
 
         # clean up events file
-        events_file and os.path.exists(events_file) and rm_rf(events_file)
+        events_file_path and os.path.exists(events_file_path) and rm_rf(events_file_path)
+        # TODO: delete events file from container!
 
         return result
 
@@ -376,7 +386,8 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
         docker_cmd = self._docker_cmd()
         if not has_been_invoked_before and config.LAMBDA_REMOTE_DOCKER:
             # if this is the first invocation: copy the entire folder into the container
-            copy_command = '%s cp "%s/." "%s:/var/task";' % (docker_cmd, lambda_cwd, container_info.name)
+            copy_command = '%s cp "%s/." "%s:%s";' % (docker_cmd,
+                lambda_cwd, container_info.name, DOCKER_TASK_FOLDER)
 
         cmd = (
             '%s'
@@ -428,57 +439,21 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
             LOG.debug('Priming docker container (status "%s"): %s' % (status, container_name))
 
             docker_image = Util.docker_image_for_lambda(func_details)
-            rm_flag = Util.get_docker_remove_flag()
 
             # Container is not running or doesn't exist.
             if status < 1:
                 # Make sure the container does not exist in any form/state.
                 self.destroy_docker_container(func_arn)
 
-                env_vars_str = ' '.join(['-e {}={}'.format(k, cmd_quote(v)) for (k, v) in env_vars])
-
-                network = config.LAMBDA_DOCKER_NETWORK
-                network_str = '--network="%s"' % network if network else ''
-
-                dns = config.LAMBDA_DOCKER_DNS
-                dns_str = '--dns="%s"' % dns if dns else ''
-
-                mount_volume = not config.LAMBDA_REMOTE_DOCKER
-                lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
-                if (':' in lambda_cwd and '\\' in lambda_cwd):
-                    lambda_cwd_on_host = Util.format_windows_path(lambda_cwd_on_host)
-                mount_volume_str = '-v "%s":/var/task' % lambda_cwd_on_host if mount_volume else ''
-
-                # Create and start the container
+                # get container startup command and run it
                 LOG.debug('Creating container: %s' % container_name)
-                cmd = (
-                    '%s create'
-                    ' %s'  # --rm flag
-                    ' --name "%s"'
-                    ' --entrypoint /bin/bash'  # Load bash when it starts.
-                    ' %s'
-                    ' --interactive'  # Keeps the container running bash.
-                    ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
-                    ' -e HOSTNAME="$HOSTNAME"'
-                    ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
-                    ' -e EDGE_PORT="$EDGE_PORT"'
-                    '  %s'  # env_vars
-                    '  %s'  # network
-                    '  %s'  # dns
-                    ' %s'
-                ) % (docker_cmd, rm_flag, container_name, mount_volume_str,
-                    env_vars_str, network_str, dns_str, docker_image)
+                cmd = self.get_container_startup_command(func_details, env_vars, lambda_cwd)
                 LOG.debug(cmd)
                 run(cmd)
 
-                if not mount_volume:
+                if config.LAMBDA_REMOTE_DOCKER:
                     LOG.debug('Copying files to container "%s" from "%s".' % (container_name, lambda_cwd))
-                    cmd = (
-                        '%s cp'
-                        ' "%s/." "%s:/var/task"'
-                    ) % (docker_cmd, lambda_cwd, container_name)
-                    LOG.debug(cmd)
-                    run(cmd)
+                    self.copy_into_container('%s/.' % lambda_cwd, container_name, DOCKER_TASK_FOLDER)
 
                 LOG.debug('Starting container: %s' % container_name)
                 cmd = '%s start %s' % (docker_cmd, container_name)
@@ -487,26 +462,78 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
                 # give the container some time to start up
                 time.sleep(1)
 
-            # Get the entry point for the image.
-            LOG.debug('Getting the entrypoint for image: %s' % (docker_image))
-            cmd = (
-                '%s image inspect'
-                ' --format="{{ .Config.Entrypoint }}"'
-                ' %s'
-            ) % (docker_cmd, docker_image)
-
-            LOG.debug(cmd)
-            run_result = run(cmd)
-
-            entry_point = run_result.strip('[]\n\r ')
-
             container_network = self.get_docker_container_network(func_arn)
+            entry_point = self.get_container_entrypoint(docker_image)
 
             LOG.debug('Using entrypoint "%s" for container "%s" on network "%s".'
                 % (entry_point, container_name, container_network))
 
             return ContainerInfo(container_name, entry_point)
 
+    def get_container_startup_command(self, func_details, env_vars, lambda_cwd):
+        docker_image = Util.docker_image_for_lambda(func_details)
+        rm_flag = Util.get_docker_remove_flag()
+        docker_cmd = self._docker_cmd()
+        container_name = self.get_container_name(func_details.arn())
+
+        env_vars_str = ' '.join(['-e {}={}'.format(k, cmd_quote(v)) for (k, v) in env_vars])
+
+        network = config.LAMBDA_DOCKER_NETWORK
+        network_str = '--network="%s"' % network if network else ''
+
+        additional_flags = config.LAMBDA_DOCKER_FLAGS or ''
+
+        dns = config.LAMBDA_DOCKER_DNS
+        dns_str = '--dns="%s"' % dns if dns else ''
+
+        mount_volume = not config.LAMBDA_REMOTE_DOCKER
+        lambda_cwd_on_host = Util.get_host_path_for_path_in_docker(lambda_cwd)
+        if (':' in lambda_cwd and '\\' in lambda_cwd):
+            lambda_cwd_on_host = Util.format_windows_path(lambda_cwd_on_host)
+        mount_volume_str = '-v "%s":%s' % (lambda_cwd_on_host, DOCKER_TASK_FOLDER) if mount_volume else ''
+
+        # Create and start the container
+        cmd = (
+            '%s create'
+            ' %s'  # --rm flag
+            ' --name "%s"'
+            ' --entrypoint /bin/bash'  # Load bash when it starts.
+            ' %s'
+            ' --interactive'  # Keeps the container running bash.
+            ' -e AWS_LAMBDA_EVENT_BODY="$AWS_LAMBDA_EVENT_BODY"'
+            ' -e HOSTNAME="$HOSTNAME"'
+            ' -e LOCALSTACK_HOSTNAME="$LOCALSTACK_HOSTNAME"'
+            ' -e EDGE_PORT="$EDGE_PORT"'
+            ' %s'  # env_vars
+            ' %s'  # network
+            ' %s'  # dns
+            ' %s'  # additional flags
+            ' %s'
+        ) % (docker_cmd, rm_flag, container_name, mount_volume_str,
+            env_vars_str, network_str, dns_str, additional_flags, docker_image)
+        return cmd
+
+    def get_container_entrypoint(self, docker_image):
+        """ Get the entry point for the given image """
+        docker_cmd = self._docker_cmd()
+        LOG.debug('Getting the entrypoint for image: %s' % (docker_image))
+        cmd = (
+            '%s image inspect'
+            ' --format="{{ .Config.Entrypoint }}"'
+            ' %s'
+        ) % (docker_cmd, docker_image)
+
+        LOG.debug(cmd)
+        run_result = run(cmd)
+
+        entry_point = run_result.strip('[]\n\r ')
+        return entry_point
+
+    def copy_into_container(self, local_path, container_name, container_path):
+        cmd = ('%s cp %s "%s:%s"') % (self._docker_cmd(), local_path, container_name, container_path)
+        LOG.debug(cmd)
+        run(cmd)
+
     def destroy_docker_container(self, func_arn):
         """
         Stops and/or removes a docker container for a specific lambda function ARN.
@@ -531,10 +558,7 @@ class LambdaExecutorReuseContainers(LambdaExecutorContainers):
 
             if status == -1:
                 LOG.debug('Removing container: %s' % container_name)
-                cmd = '%s rm %s' % (docker_cmd, container_name)
-
-                LOG.debug(cmd)
-                run(cmd, asynchronous=False, stderr=subprocess.PIPE, outfile=subprocess.PIPE)
+                rm_docker_container(container_name, safe=True)
 
     def get_all_container_names(self):
         """
@@ -697,6 +721,8 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
             env_vars['DOCKER_LAMBDA_API_PORT'] = port
             env_vars['DOCKER_LAMBDA_RUNTIME_PORT'] = port
 
+        additional_flags = config.LAMBDA_DOCKER_FLAGS or ''
+
         dns = config.LAMBDA_DOCKER_DNS
         dns_str = '--dns="%s"' % dns if dns else ''
 
@@ -706,40 +732,35 @@ class LambdaExecutorSeparateContainers(LambdaExecutorContainers):
         docker_image = Util.docker_image_for_lambda(func_details)
         rm_flag = Util.get_docker_remove_flag()
 
+        # construct common flags for commands below
+        common_flags = ' '.join([env_vars_string, network_str, dns_str, additional_flags, rm_flag])
+
         if config.LAMBDA_REMOTE_DOCKER:
-            cp_cmd = '%s cp "%s/." "$CONTAINER_ID:/var/task";' % (docker_cmd, lambda_cwd) if lambda_cwd else ''
+            cp_cmd = ('%s cp "%s/." "$CONTAINER_ID:%s";' % (
+                docker_cmd, lambda_cwd, DOCKER_TASK_FOLDER)) if lambda_cwd else ''
             cmd = (
                 'CONTAINER_ID="$(%s create -i'
                 ' %s'  # entrypoint
                 ' %s'  # debug_docker_java_port
-                ' %s'  # env
-                ' %s'  # network
-                ' %s'  # dns
-                ' %s'  # --rm flag
+                ' %s'  # common flags
                 ' %s %s'  # image and command
                 ')";'
                 '%s '
                 '%s start -ai "$CONTAINER_ID";'
             ) % (docker_cmd, entrypoint, debug_docker_java_port,
-                 env_vars_string, network_str, dns_str, rm_flag,
-                 docker_image, command,
-                 cp_cmd,
-                 docker_cmd)
+                 common_flags, docker_image, command,
+                 cp_cmd, docker_cmd)
         else:
             mount_flag = ''
             if lambda_cwd:
-                mount_flag = '-v "%s":/var/task' % Util.get_host_path_for_path_in_docker(lambda_cwd)
+                mount_flag = '-v "%s":%s' % (Util.get_host_path_for_path_in_docker(lambda_cwd), DOCKER_TASK_FOLDER)
             cmd = (
                 '%s run -i'
                 ' %s'
                 ' %s'  # code mount
-                ' %s'
-                ' %s'  # network
-                ' %s'  # dns
-                ' %s'  # --rm flag
+                ' %s'  # common flags
                 ' %s %s'
-            ) % (docker_cmd, entrypoint, mount_flag, env_vars_string,
-                 network_str, dns_str, rm_flag, docker_image, command)
+            ) % (docker_cmd, entrypoint, mount_flag, common_flags, docker_image, command)
         return cmd
 
 
@@ -755,12 +776,11 @@ class LambdaExecutorLocal(LambdaExecutor):
 
         def do_execute():
             # now we're executing in the child process, safe to change CWD and ENV
-            path_before = sys.path
             result = None
             try:
                 if lambda_cwd:
                     os.chdir(lambda_cwd)
-                    sys.path = [lambda_cwd] + sys.path
+                    sys.path.insert(0, '')
                 if environment:
                     os.environ.update(environment)
                 result = lambda_function(event, context)
@@ -769,7 +789,6 @@ class LambdaExecutorLocal(LambdaExecutor):
                 sys.stderr.write('%s %s' % (e, traceback.format_exc()))
                 raise
             finally:
-                sys.path = path_before
                 queue.put(result)
 
         process = Process(target=do_execute)
