@@ -4,14 +4,14 @@ import traceback
 import xmltodict
 from flask import Flask, request
 from requests.models import Response
-from localstack.utils.aws import aws_stack
+from localstack.utils.aws import aws_stack, aws_responses
 from localstack.utils.common import (
     parse_request_data, short_uid, long_uid, clone, clone_safe, select_attributes,
     timestamp_millis, recurse_object)
 from localstack.utils.cloudformation import template_deployer, template_preparer
 from localstack.services.generic_proxy import RegionBackend
 from localstack.utils.aws.aws_responses import (
-    requests_response_xml, requests_to_flask_response, flask_error_response_xml)
+    requests_response_xml, requests_to_flask_response, flask_error_response_xml, extract_url_encoded_param_list)
 
 APP_NAME = 'cloudformation_api'
 app = Flask(APP_NAME)
@@ -26,6 +26,8 @@ class CloudFormationRegion(RegionBackend):
     def __init__(self):
         # maps stack ID to stack details
         self.stacks = {}
+        # maps stack set ID to stack set details
+        self.stack_sets = {}
 
     @property
     def exports(self):
@@ -50,6 +52,28 @@ class CloudFormationRegion(RegionBackend):
         return exports
 
 
+class StackSet(object):
+    """ A stack set contains multiple stack instances. """
+    def __init__(self, metadata={}):
+        self.metadata = metadata
+        # list of stack instances
+        self.stack_instances = []
+        # maps operation ID to stack set operation details
+        self.operations = {}
+
+    @property
+    def stack_set_name(self):
+        return self.metadata.get('StackSetName')
+
+
+class StackInstance(object):
+    """ A stack instance belongs to a stack set and is specific to a region / account ID. """
+    def __init__(self, metadata={}):
+        self.metadata = metadata
+        # reference to the deployed stack belonging to this stack instance
+        self.stack = None
+
+
 class Stack(object):
     def __init__(self, metadata=None, template={}):
         self.metadata = metadata or {}
@@ -101,6 +125,7 @@ class Stack(object):
 
     def set_stack_status(self, status):
         self.metadata['StackStatus'] = status
+        self.metadata['StackStatusReason'] = 'Deployment %s' % ('failed' if 'FAILED' in status else 'succeeded')
         self.add_stack_event(self.stack_name, self.stack_id, status)
 
     def add_stack_event(self, resource_id, physical_res_id, status):
@@ -177,7 +202,7 @@ class Stack(object):
 
     @property
     def tags(self):
-        return aws_stack.extract_tags(self.metadata)
+        return aws_responses.extract_tags(self.metadata)
 
     @property
     def imports(self):
@@ -207,6 +232,7 @@ class Stack(object):
                 LOG.debug('Unable to resolve references in stack outputs: %s - %s' % (details, e))
             exports = details.get('Export') or {}
             export = exports.get('Name')
+            export = template_deployer.resolve_refs_recursively(self.stack_name, export, self.resources)
             description = details.get('Description')
             entry = {'OutputKey': k, 'OutputValue': value, 'Description': description, 'ExportName': export}
             result.append(entry)
@@ -338,6 +364,65 @@ def create_stack(req_params):
     return result
 
 
+def create_stack_set(req_params):
+    state = CloudFormationRegion.get()
+    stack_set = StackSet(req_params)
+    stack_set_id = short_uid()
+    stack_set.metadata['StackSetId'] = stack_set_id
+    state.stack_sets[stack_set_id] = stack_set
+    result = {'StackSetId': stack_set_id}
+    return result
+
+
+def create_stack_instances(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not stack_set:
+        return not_found_error('Stack set named "%s" does not exist' % set_name)
+    stack_set = stack_set[0]
+    op_id = req_params.get('OperationId') or short_uid()
+    sset_meta = stack_set.metadata
+    accounts = extract_url_encoded_param_list(req_params, 'Accounts.member.%s')
+    accounts = accounts or extract_url_encoded_param_list(req_params, 'DeploymentTargets.Accounts.member.%s')
+    regions = extract_url_encoded_param_list(req_params, 'Regions.member.%s')
+    stacks_to_await = []
+    for account in accounts:
+        for region in regions:
+            # deploy new stack
+            LOG.debug('Deploying instance for stack set "%s" in region "%s"' % (set_name, region))
+            cf_client = aws_stack.connect_to_service('cloudformation', region_name=region)
+            kwargs = select_attributes(sset_meta, 'TemplateBody') or select_attributes(sset_meta, 'TemplateURL')
+            stack_name = 'sset-%s-%s' % (set_name, account)
+            result = cf_client.create_stack(StackName=stack_name, **kwargs)
+            stacks_to_await.append((stack_name, region))
+            # store stack instance
+            instance = {
+                'StackSetId': sset_meta['StackSetId'],
+                'OperationId': op_id,
+                'Account': account,
+                'Region': region,
+                'StackId': result['StackId'],
+                'Status': 'CURRENT',
+                'StackInstanceStatus': {'DetailedStatus': 'SUCCEEDED'}
+            }
+            instance = StackInstance(instance)
+            stack_set.stack_instances.append(instance)
+    # wait for completion of stack
+    for stack in stacks_to_await:
+        aws_stack.await_stack_completion(stack[0], region_name=stack[1])
+    # record operation
+    operation = {
+        'OperationId': op_id,
+        'StackSetId': stack_set.metadata['StackSetId'],
+        'Action': 'CREATE',
+        'Status': 'SUCCEEDED'
+    }
+    stack_set.operations[op_id] = operation
+    result = {'OperationId': op_id}
+    return result
+
+
 def delete_stack(req_params):
     stack_name = req_params.get('StackName')
     stack = find_stack(stack_name)
@@ -346,12 +431,23 @@ def delete_stack(req_params):
     return {}
 
 
+def delete_stack_set(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not stack_set:
+        return not_found_error('Stack set named "%s" does not exist' % set_name)
+    for instance in stack_set[0].stack_instances:
+        deployer = template_deployer.TemplateDeployer(instance.stack)
+        deployer.delete_stack()
+    return {}
+
+
 def update_stack(req_params):
     stack_name = req_params.get('StackName')
     stack = find_stack(stack_name)
     if not stack:
-        return error_response('Unable to update non-existing stack "%s"' % stack_name,
-            code=404, code_string='ValidationError')
+        return not_found_error('Unable to update non-existing stack "%s"' % stack_name)
     template_preparer.prepare_template_body(req_params)
     template = template_preparer.parse_template(req_params['TemplateBody'])
     new_stack = Stack(req_params, template)
@@ -367,10 +463,30 @@ def update_stack(req_params):
     return result
 
 
+def update_stack_set(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not stack_set:
+        return not_found_error('Stack set named "%s" does not exist' % set_name)
+    stack_set = stack_set[0]
+    stack_set.metadata.update(req_params)
+    op_id = req_params.get('OperationId') or short_uid()
+    operation = {
+        'OperationId': op_id,
+        'StackSetId': stack_set.metadata['StackSetId'],
+        'Action': 'UPDATE',
+        'Status': 'SUCCEEDED'
+    }
+    stack_set.operations[op_id] = operation
+    return {'OperationId': op_id}
+
+
 def describe_stacks(req_params):
     state = CloudFormationRegion.get()
     stack_name = req_params.get('StackName')
-    stacks = [s.describe_details() for s in state.stacks.values() if stack_name in [None, s.stack_name, s.stack_id]]
+    stack_list = list(state.stacks.values())
+    stacks = [s.describe_details() for s in stack_list if stack_name in [None, s.stack_name, s.stack_id]]
     if stack_name and not stacks:
         return error_response('Stack with id %s does not exist' % stack_name,
             code=400, code_string='ValidationError')
@@ -380,10 +496,14 @@ def describe_stacks(req_params):
 
 def list_stacks(req_params):
     state = CloudFormationRegion.get()
-    filter = req_params.get('StackStatusFilter')
-    stacks = [s.describe_details() for s in state.stacks.values() if filter in [None, s.status]]
+
+    stack_status_filters = _get_status_filter_members(req_params)
+
+    stacks = [s.describe_details() for s in state.stacks.values() if
+        not stack_status_filters or s.status in stack_status_filters]
+
     attrs = ['StackId', 'StackName', 'TemplateDescription', 'CreationTime', 'LastUpdatedTime', 'DeletionTime',
-        'StackStatus', 'StackStatusReason', 'ParentId', 'RootId', 'DriftInformation']
+             'StackStatus', 'StackStatusReason', 'ParentId', 'RootId', 'DriftInformation']
     stacks = [select_attributes(stack, attrs) for stack in stacks]
     result = {'StackSummaries': {'member': stacks}}
     return result
@@ -405,7 +525,7 @@ def describe_stack_resources(req_params):
     resource_id = req_params.get('LogicalResourceId')
     phys_resource_id = req_params.get('PhysicalResourceId')
     if phys_resource_id and stack_name:
-        return error_response('Cannot specify both StackName and PhysicalResourceId')
+        return error_response('Cannot specify both StackName and PhysicalResourceId', code=400)
     # TODO: filter stack by PhysicalResourceId!
     stack = find_stack(stack_name)
     if not stack:
@@ -423,6 +543,18 @@ def list_stack_resources(req_params):
     return result
 
 
+def list_stack_instances(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not stack_set:
+        return not_found_error('Stack set named "%s" does not exist' % set_name)
+    stack_set = stack_set[0]
+    result = [inst.metadata for inst in stack_set.stack_instances]
+    result = {'Summaries': {'member': result}}
+    return result
+
+
 def create_change_set(req_params):
     stack_name = req_params.get('StackName')
     template_deployer.prepare_template_body(req_params)
@@ -444,6 +576,8 @@ def create_change_set(req_params):
     deployer.construct_changes(stack, change_set, change_set_id=change_set.change_set_id, append_to_changeset=True)
     stack.change_sets.append(change_set)
     change_set.metadata['Status'] = 'CREATE_COMPLETE'
+    change_set.metadata['ExecutionStatus'] = 'AVAILABLE'
+    change_set.metadata['StatusReason'] = 'Changeset created'
     return {'StackId': change_set.stack_id, 'Id': change_set.change_set_id}
 
 
@@ -452,7 +586,7 @@ def execute_change_set(req_params):
     cs_name = req_params.get('ChangeSetName')
     change_set = find_change_set(cs_name, stack_name=stack_name)
     if not change_set:
-        return error_response('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
+        return not_found_error('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
     LOG.debug('Executing change set "%s" for stack "%s" with %s resources ...' % (
         cs_name, stack_name, len(change_set.template_resources)))
     deployer = template_deployer.TemplateDeployer(change_set.stack)
@@ -465,21 +599,55 @@ def list_change_sets(req_params):
     stack_name = req_params.get('StackName')
     stack = find_stack(stack_name)
     if not stack:
-        return error_response('Unable to find stack "%s"' % stack_name)
+        return not_found_error('Unable to find stack "%s"' % stack_name)
     result = [cs.metadata for cs in stack.change_sets]
     result = {'Summaries': {'member': result}}
     return result
 
 
+def list_stack_sets(req_params):
+    state = CloudFormationRegion.get()
+    result = [sset.metadata for sset in state.stack_sets.values()]
+    result = {'Summaries': {'member': result}}
+    return result
+
+
 def describe_change_set(req_params):
     stack_name = req_params.get('StackName')
     cs_name = req_params.get('ChangeSetName')
     change_set = find_change_set(cs_name, stack_name=stack_name)
     if not change_set:
-        return error_response('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
+        return not_found_error('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
     return change_set.metadata
 
 
+def describe_stack_set(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    result = [sset.metadata for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not result:
+        return not_found_error('Unable to find stack set "%s"' % set_name)
+    result = {'StackSet': result[0]}
+    return result
+
+
+def describe_stack_set_operation(req_params):
+    state = CloudFormationRegion.get()
+    set_name = req_params.get('StackSetName')
+    stack_set = [sset for sset in state.stack_sets.values() if sset.stack_set_name == set_name]
+    if not stack_set:
+        return not_found_error('Unable to find stack set "%s"' % set_name)
+    stack_set = stack_set[0]
+    op_id = req_params.get('OperationId')
+    result = stack_set.operations.get(op_id)
+    if not result:
+        LOG.debug('Unable to find operation ID "%s" for stack set "%s" in list: %s' % (
+            op_id, set_name, list(stack_set.operations.keys())))
+        return not_found_error('Unable to find operation ID "%s" for stack set "%s"' % (op_id, set_name))
+    result = {'StackSetOperation': result}
+    return result
+
+
 def list_exports(req_params):
     state = CloudFormationRegion.get()
     result = {'Exports': {'member': state.exports}}
@@ -522,7 +690,7 @@ def delete_change_set(req_params):
     cs_name = req_params.get('ChangeSetName')
     change_set = find_change_set(cs_name, stack_name=stack_name)
     if not change_set:
-        return error_response('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
+        return not_found_error('Unable to find change set "%s" for stack "%s"' % (cs_name, stack_name))
     change_set.stack.change_sets = [cs for cs in change_set.stack.change_sets if cs.change_set_name != cs_name]
     return {}
 
@@ -585,22 +753,30 @@ def handle_request():
 ENDPOINTS = {
     'CreateChangeSet': create_change_set,
     'CreateStack': create_stack,
+    'CreateStackInstances': create_stack_instances,
+    'CreateStackSet': create_stack_set,
     'DeleteChangeSet': delete_change_set,
     'DeleteStack': delete_stack,
+    'DeleteStackSet': delete_stack_set,
     'DescribeChangeSet': describe_change_set,
     'DescribeStackEvents': describe_stack_events,
     'DescribeStackResource': describe_stack_resource,
     'DescribeStackResources': describe_stack_resources,
     'DescribeStacks': describe_stacks,
+    'DescribeStackSet': describe_stack_set,
+    'DescribeStackSetOperation': describe_stack_set_operation,
     'ExecuteChangeSet': execute_change_set,
     'GetTemplate': get_template,
     'GetTemplateSummary': get_template_summary,
     'ListChangeSets': list_change_sets,
     'ListExports': list_exports,
     'ListImports': list_imports,
+    'ListStackInstances': list_stack_instances,
     'ListStacks': list_stacks,
     'ListStackResources': list_stack_resources,
+    'ListStackSets': list_stack_sets,
     'UpdateStack': update_stack,
+    'UpdateStackSet': update_stack_set,
     'ValidateTemplate': validate_template
 }
 
@@ -614,9 +790,12 @@ def error_response(*args, **kwargs):
     return flask_error_response_xml(*args, **kwargs)
 
 
+def not_found_error(message):
+    return error_response(message, code=404, code_string='ResourceNotFoundException')
+
+
 def stack_not_found_error(stack_name):
-    return error_response('Unable to find stack named "%s"' % stack_name,
-        code=404, code_string='ResourceNotFoundException')
+    return not_found_error('Unable to find stack named "%s"' % stack_name)
 
 
 def clone_stack_params(stack_params):
@@ -651,3 +830,27 @@ def _response(action, result):
 def serve(port, quiet=True):
     from localstack.services import generic_proxy  # moved here to fix circular import errors
     return generic_proxy.serve_flask_app(app=app, port=port, quiet=quiet)
+
+
+def _get_status_filter_members(req_params):
+    """
+    Creates a set of status from the requests parameters
+
+    The API request params specify two parameters of the endpoint:
+       - NextToken: Token for next page
+       - StackStatusFilter.member.N: Status to use as a filter (it conforms a list)
+
+    StackStatusFilter.member.N abstracts the list of status in the request, and it is sent
+    as different parameters, as the N suggests.
+
+    Docs:
+         https://docs.aws.amazon.com/AWSCloudFormation/latest/APIReference/API_ListStacks.html
+
+    Returns:
+        set: set of status to filter upon
+    """
+    return {
+        value
+        for param, value in req_params.items()
+        if param.startswith('StackStatusFilter.member')
+    }
