@@ -1,3 +1,4 @@
+import os
 import re
 import json
 import logging
@@ -8,17 +9,23 @@ from moto.iam.models import Role as MotoRole
 from moto.core.models import CloudFormationModel
 from moto.cloudformation.exceptions import UnformattedGetAttTemplateException
 from localstack.constants import AWS_REGION_US_EAST_1, LOCALHOST
+from localstack.utils import common
 from localstack.utils.aws import aws_stack
-from localstack.utils.common import camel_to_snake_case, select_attributes, canonical_json, md5
+from localstack.utils.common import (
+    camel_to_snake_case, select_attributes, canonical_json, md5, is_base64,
+    new_tmp_dir, save_file, rm_rf, mkdir, cp_r, short_uid)
+from localstack.utils.testutil import create_zip_file
+from localstack.services.awslambda.lambda_api import (
+    get_handler_file_from_name, LAMBDA_POLICY_NAME_PATTERN)
 from localstack.services.cloudformation.deployment_utils import (
     PLACEHOLDER_RESOURCE_NAME, remove_none_values, params_list_to_dict, lambda_keys_to_lower,
     merge_parameters, params_dict_to_list, select_parameters, params_select_attributes,
-    lambda_select_params)
+    lambda_select_params, get_cfn_response_mod_file)
 
 LOG = logging.getLogger(__name__)
 
-# name pattern of IAM policies associated with Lambda functions
-LAMBDA_POLICY_NAME_PATTERN = 'lambda_policy_%s'
+# dict key used to store the deployment state of a resource
+KEY_RESOURCE_STATE = '_state_'
 
 # ref attribute definitions
 REF_ATTRS = ['PhysicalResourceId', 'Ref']
@@ -50,11 +57,11 @@ class GenericBaseModel(CloudFormationModel):
         self.resource_json = resource_json
         self.resource_type = resource_json['Type']
         # Properties, as defined in the resource template
-        self.properties = resource_json.get('Properties') or {}
+        self.properties = resource_json['Properties'] = resource_json.get('Properties') or {}
         # State, as determined from the deployed resource; use a special dict key here to keep
         # track of state changes within resource_json (this way we encapsulate all state details
         # in `resource_json` and the changes will survive creation of multiple instances of this class)
-        self.state = resource_json['_state_'] = resource_json.get('_state_') or {}
+        self.state = resource_json[KEY_RESOURCE_STATE] = resource_json.get(KEY_RESOURCE_STATE) or {}
 
     # ----------------------
     # ABSTRACT BASE METHODS
@@ -108,9 +115,9 @@ class GenericBaseModel(CloudFormationModel):
 
         raise UnformattedGetAttTemplateException()
 
-    # ----------------------
+    # ---------------------
     # GENERIC UTIL METHODS
-    # ----------------------
+    # ---------------------
 
     def fetch_and_update_state(self, *args, **kwargs):
         from localstack.utils.cloudformation import template_deployer
@@ -202,6 +209,46 @@ class EventsRule(GenericBaseModel):
         result = aws_stack.connect_to_service('events').describe_rule(Name=rule_name) or {}
         return result if result.get('Name') else None
 
+    @classmethod
+    def get_deploy_templates(cls):
+        def events_put_rule_params(params, **kwargs):
+            attrs = ['ScheduleExpression', 'EventPattern', 'State', 'Description', 'Name', 'EventBusName']
+            result = select_parameters(*attrs)(params, **kwargs)
+            result['Name'] = result.get('Name') or PLACEHOLDER_RESOURCE_NAME
+
+            def wrap_in_lists(o, **kwargs):
+                if isinstance(o, dict):
+                    for k, v in o.items():
+                        if not isinstance(v, (dict, list)):
+                            o[k] = [v]
+                return o
+
+            pattern = result.get('EventPattern')
+            if isinstance(pattern, dict):
+                wrapped = common.recurse_object(pattern, wrap_in_lists)
+                result['EventPattern'] = json.dumps(wrapped)
+            return result
+
+        return {
+            'create': [{
+                'function': 'put_rule',
+                'parameters': events_put_rule_params
+            }, {
+                'function': 'put_targets',
+                'parameters': {
+                    'Rule': PLACEHOLDER_RESOURCE_NAME,
+                    'EventBusName': 'EventBusName',
+                    'Targets': 'Targets'
+                }
+            }],
+            'delete': {
+                'function': 'delete_rule',
+                'parameters': {
+                    'Name': 'PhysicalResourceId'
+                }
+            }
+        }
+
 
 class EventBus(GenericBaseModel):
     @staticmethod
@@ -222,10 +269,16 @@ class LogsLogGroup(GenericBaseModel):
         return 'AWS::Logs::LogGroup'
 
     def get_cfn_attribute(self, attribute_name):
+        props = self.props
         if attribute_name == 'Arn':
-            return self.params.get('Arn') or aws_stack.log_group_arn(self.params.get('LogGroupName'))
+            return props.get('arn')
         return super(LogsLogGroup, self).get_cfn_attribute(attribute_name)
 
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        if attribute == 'Arn':
+            return self.get_cfn_attribute('Arn')
+        return self.props.get('LogGroupName')
+
     def fetch_state(self, stack_name, resources):
         group_name = self.props.get('LogGroupName')
         group_name = self.resolve_refs_recursively(stack_name, group_name, resources)
@@ -233,6 +286,62 @@ class LogsLogGroup(GenericBaseModel):
         groups = logs.describe_log_groups(logGroupNamePrefix=group_name)['logGroups']
         return ([g for g in groups if g['logGroupName'] == group_name] or [None])[0]
 
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_log_group',
+                'parameters': {
+                    'logGroupName': 'LogGroupName'
+                }
+            },
+            'delete': {
+                'function': 'delete_log_group',
+                'parameters': {
+                    'logGroupName': 'LogGroupName'
+                }
+            }
+        }
+
+
+class LogsSubscriptionFilter(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::Logs::SubscriptionFilter'
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('LogGroupName')
+
+    def fetch_state(self, stack_name, resources):
+        props = self.props
+        group_name = self.resolve_refs_recursively(stack_name, props.get('LogGroupName'), resources)
+        filter_pattern = self.resolve_refs_recursively(stack_name, props.get('FilterPattern'), resources)
+        logs = aws_stack.connect_to_service('logs')
+        groups = logs.describe_subscription_filters(logGroupName=group_name)['subscriptionFilters']
+        groups = [g for g in groups if g.get('filterPattern') == filter_pattern]
+        return (groups or [None])[0]
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'put_subscription_filter',
+                'parameters': {
+                    'logGroupName': 'LogGroupName',
+                    'filterName': 'LogGroupName',  # there can only be one filter associated with a log group
+                    'filterPattern': 'FilterPattern',
+                    'destinationArn': 'DestinationArn'
+                }
+            },
+            'delete': {
+                'function': 'delete_subscription_filter',
+                'parameters': {
+                    'logGroupName': 'LogGroupName',
+                    'filterName': 'LogGroupName'
+                }
+            }
+        }
+
 
 class CloudFormationStack(GenericBaseModel):
     @staticmethod
@@ -305,6 +414,63 @@ class LambdaFunction(GenericBaseModel):
             update_props['Environment']['Variables'] = {k: str(v) for k, v in environment_variables.items()}
         return client.update_function_configuration(**update_props)
 
+    @staticmethod
+    def get_deploy_templates():
+        def get_lambda_code_param(params, **kwargs):
+            code = params.get('Code', {})
+            zip_file = code.get('ZipFile')
+            if zip_file and not is_base64(zip_file):
+                tmp_dir = new_tmp_dir()
+                handler_file = get_handler_file_from_name(params['Handler'], runtime=params['Runtime'])
+                tmp_file = os.path.join(tmp_dir, handler_file)
+                save_file(tmp_file, zip_file)
+
+                # add 'cfn-response' module to archive - see:
+                # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cfn-lambda-function-code-cfnresponsemodule.html
+                cfn_response_tmp_file = get_cfn_response_mod_file()
+                cfn_response_mod_dir = os.path.join(tmp_dir, 'node_modules', 'cfn-response')
+                mkdir(cfn_response_mod_dir)
+                cp_r(cfn_response_tmp_file, os.path.join(cfn_response_mod_dir, 'index.js'))
+
+                # create zip file
+                zip_file = create_zip_file(tmp_dir, get_content=True)
+                code['ZipFile'] = zip_file
+                rm_rf(tmp_dir)
+            return code
+
+        def get_delete_params(params, **kwargs):
+            return {'FunctionName': params.get('FunctionName')}
+
+        return {
+            'create': {
+                'function': 'create_function',
+                'parameters': {
+                    'FunctionName': 'FunctionName',
+                    'Runtime': 'Runtime',
+                    'Role': 'Role',
+                    'Handler': 'Handler',
+                    'Code': get_lambda_code_param,
+                    'Description': 'Description',
+                    'Environment': 'Environment',
+                    'Timeout': 'Timeout',
+                    'MemorySize': 'MemorySize',
+                    'Layers': 'Layers'
+                    # TODO add missing fields
+                },
+                'defaults': {
+                    'Role': 'test_role'
+                },
+                'types': {
+                    'Timeout': int,
+                    'MemorySize': int
+                }
+            },
+            'delete': {
+                'function': 'delete_function',
+                'parameters': get_delete_params
+            }
+        }
+
 
 class LambdaFunctionVersion(GenericBaseModel):
     @staticmethod
@@ -353,19 +519,24 @@ class LambdaPermission(GenericBaseModel):
         return 'AWS::Lambda::Permission'
 
     def fetch_state(self, stack_name, resources):
+        props = self.props
+        func_name = self.resolve_refs_recursively(stack_name, props.get('FunctionName'), resources)
+        func_arn = aws_stack.lambda_function_arn(func_name)
+        return self.do_fetch_state(func_name, func_arn)
+
+    def do_fetch_state(self, resource_name, resource_arn):
         iam = aws_stack.connect_to_service('iam')
         props = self.props
-        policy_name = LAMBDA_POLICY_NAME_PATTERN % props.get('FunctionName')
+        policy_name = LAMBDA_POLICY_NAME_PATTERN % resource_name
         policy_arn = aws_stack.policy_arn(policy_name)
         policy = iam.get_policy(PolicyArn=policy_arn)['Policy']
         version = policy.get('DefaultVersionId')
         policy = iam.get_policy_version(PolicyArn=policy_arn, VersionId=version)['PolicyVersion']
         statements = policy['Document']['Statement']
         statements = statements if isinstance(statements, list) else [statements]
-        func_arn = aws_stack.lambda_function_arn(props['FunctionName'])
         principal = props.get('Principal')
         existing = [s for s in statements if s['Action'] == props['Action'] and
-            s['Resource'] == func_arn and
+            s['Resource'] == resource_arn and
             (not principal or s['Principal'] in [principal, {'Service': principal}, {'Service': [principal]}])]
         return existing[0] if existing else None
 
@@ -373,6 +544,19 @@ class LambdaPermission(GenericBaseModel):
         # return statement ID here to indicate that the resource has been deployed
         return self.props.get('Sid')
 
+    @staticmethod
+    def get_deploy_templates():
+        def lambda_permission_params(params, **kwargs):
+            result = select_parameters('FunctionName', 'Action', 'Principal')(params, **kwargs)
+            result['StatementId'] = short_uid()
+            return result
+        return {
+            'create': {
+                'function': 'add_permission',
+                'parameters': lambda_permission_params
+            }
+        }
+
 
 class LambdaEventInvokeConfig(GenericBaseModel):
     @staticmethod
@@ -686,6 +870,45 @@ class GatewayResponse(GenericBaseModel):
         return result if 'responseType' in result else None
 
 
+class GatewayRequestValidator(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::ApiGateway::RequestValidator'
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('id')
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('apigateway')
+        props = self.props
+        api_id = self.resolve_refs_recursively(stack_name, props['RestApiId'], resources)
+        name = self.resolve_refs_recursively(stack_name, props['Name'], resources)
+        result = client.get_request_validators(restApiId=api_id).get('items', [])
+        result = [r for r in result if r.get('name') == name]
+        return result[0] if result else None
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_request_validator',
+                'parameters': {
+                    'name': 'Name',
+                    'restApiId': 'RestApiId',
+                    'validateRequestBody': 'ValidateRequestBody',
+                    'validateRequestParameters': 'ValidateRequestParameters'
+                }
+            },
+            'delete': {
+                'function': 'delete_request_validator',
+                'parameters': {
+                    'restApiId': 'RestApiId',
+                    'requestValidatorId': 'id'
+                }
+            }
+        }
+
+
 class GatewayRestAPI(GenericBaseModel):
     @staticmethod
     def cloudformation_type():
@@ -732,10 +955,13 @@ class GatewayDeployment(GenericBaseModel):
     def fetch_state(self, stack_name, resources):
         api_id = self.props.get('RestApiId') or self.resource_id
         api_id = self.resolve_refs_recursively(stack_name, api_id, resources)
+
         if not api_id:
             return None
+
         result = aws_stack.connect_to_service('apigateway').get_deployments(restApiId=api_id)['items']
         # TODO possibly filter results by stage name or other criteria
+
         return result[0] if result else None
 
 
@@ -749,15 +975,18 @@ class GatewayResource(GenericBaseModel):
         api_id = props.get('RestApiId') or self.resource_id
         api_id = self.resolve_refs_recursively(stack_name, api_id, resources)
         parent_id = self.resolve_refs_recursively(stack_name, props.get('ParentId'), resources)
+
         if not api_id or not parent_id:
             return None
+
         api_resources = aws_stack.connect_to_service('apigateway').get_resources(restApiId=api_id)['items']
         target_resource = list(filter(lambda res:
             res.get('parentId') == parent_id and res['pathPart'] == props['PathPart'], api_resources))
+
         if not target_resource:
             return None
-        path = aws_stack.get_apigateway_path_for_resource(api_id,
-            target_resource[0]['id'], resources=api_resources)
+
+        path = aws_stack.get_apigateway_path_for_resource(api_id, target_resource[0]['id'], resources=api_resources)
         result = list(filter(lambda res: res['path'] == path, api_resources))
         return result[0] if result else None
 
@@ -769,18 +998,22 @@ class GatewayMethod(GenericBaseModel):
 
     def fetch_state(self, stack_name, resources):
         props = self.props
+
         api_id = self.resolve_refs_recursively(stack_name, props['RestApiId'], resources)
         res_id = self.resolve_refs_recursively(stack_name, props['ResourceId'], resources)
         if not api_id or not res_id:
             return None
+
         res_obj = aws_stack.connect_to_service('apigateway').get_resource(restApiId=api_id, resourceId=res_id)
         match = [v for (k, v) in res_obj.get('resourceMethods', {}).items()
                  if props['HttpMethod'] in (v.get('httpMethod'), k)]
+
         int_props = props.get('Integration') or {}
         if int_props.get('Type') == 'AWS_PROXY':
-            match = [m for m in match if
-                m.get('methodIntegration', {}).get('type') == 'AWS_PROXY' and
-                m.get('methodIntegration', {}).get('httpMethod') == int_props.get('IntegrationHttpMethod')]
+            match = [m for m in match
+                     if m.get('methodIntegration', {}).get('type') == 'AWS_PROXY' and
+                     m.get('methodIntegration', {}).get('httpMethod') == int_props.get('IntegrationHttpMethod')]
+
         return match[0] if match else None
 
     def update_resource(self, new_resource, stack_name, resources):
@@ -929,6 +1162,21 @@ class GatewayModel(GenericBaseModel):
     def cloudformation_type():
         return 'AWS::ApiGateway::Model'
 
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('apigateway')
+        api_id = self.resolve_refs_recursively(stack_name, self.props['RestApiId'], resources)
+
+        items = client.get_models(restApiId=api_id)['items']
+        if not items:
+            return None
+
+        model_name = self.resolve_refs_recursively(stack_name, self.props['Name'], resources)
+        models = [item for item in items if item['name'] == model_name]
+        if models:
+            return models[0]
+
+        return None
+
 
 class GatewayAccount(GenericBaseModel):
     @staticmethod
@@ -949,7 +1197,6 @@ class S3Bucket(GenericBaseModel, FakeBucket):
 
     @staticmethod
     def get_deploy_templates():
-
         def convert_acl_cf_to_s3(acl):
             """ Convert a CloudFormation ACL string (e.g., 'PublicRead') to an S3 ACL string (e.g., 'public-read') """
             return re.sub('(?<!^)(?=[A-Z])', '-', acl).lower()
@@ -1211,6 +1458,14 @@ class SNSSubscription(GenericBaseModel):
         }
 
 
+class QueuePolicy(GenericBaseModel):
+    @staticmethod
+    def cloudformation_type():
+        return 'AWS::SQS::QueuePolicy'
+
+    # TODO: add deployment methods
+
+
 class DynamoDBTable(GenericBaseModel):
     @staticmethod
     def cloudformation_type():
@@ -1228,10 +1483,27 @@ class DynamoDBTable(GenericBaseModel):
         return aws_stack.connect_to_service('dynamodb').describe_table(TableName=table_name)
 
 
-class QueuePolicy(GenericBaseModel):
+class RedshiftCluster(GenericBaseModel):
     @staticmethod
     def cloudformation_type():
-        return 'AWS::SQS::QueuePolicy'
+        return 'AWS::Redshift::Cluster'
+
+    def get_physical_resource_id(self, attribute=None, **kwargs):
+        return self.props.get('ClusterIdentifier')
+
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('redshift')
+        cluster_id = self.resolve_refs_recursively(stack_name, self.props.get('ClusterIdentifier'), resources)
+        result = client.describe_clusters(ClusterIdentifier=cluster_id)['Clusters']
+        return (result or [None])[0]
+
+    @staticmethod
+    def get_deploy_templates():
+        return {
+            'create': {
+                'function': 'create_cluster'
+            }
+        }
 
 
 class SSMParameter(GenericBaseModel):
@@ -1717,6 +1989,20 @@ class EC2Route(GenericBaseModel):
     def cloudformation_type():
         return 'AWS::EC2::Route'
 
+    def fetch_state(self, stack_name, resources):
+        client = aws_stack.connect_to_service('ec2')
+        props = self.props
+        dst_cidr = self.resolve_refs_recursively(stack_name, props.get('DestinationCidrBlock'), resources)
+        dst_cidr6 = self.resolve_refs_recursively(stack_name, props.get('DestinationIpv6CidrBlock'), resources)
+        table_id = self.resolve_refs_recursively(stack_name, props.get('RouteTableId'), resources)
+        route_tables = client.describe_route_tables()['RouteTables']
+        route_table = ([t for t in route_tables if t['RouteTableId'] == table_id] or [None])[0]
+        if route_table:
+            routes = route_table.get('Routes', [])
+            route = [r for r in routes if r.get('DestinationCidrBlock') == (dst_cidr or '_not_set_') or
+                r.get('DestinationIpv6CidrBlock') == (dst_cidr6 or '_not_set_')]
+            return (route or [None])[0]
+
     def get_physical_resource_id(self, attribute=None, **kwargs):
         props = self.props
         return generate_route_id(
@@ -1756,7 +2042,7 @@ class EC2InternetGateway(GenericBaseModel):
         client = aws_stack.connect_to_service('ec2')
         gateways = client.describe_internet_gateways()['InternetGateways']
         tags = self.props.get('Tags')
-        gateway = [g for g in gateways if g.get('Tags') == tags]
+        gateway = [g for g in gateways if (g.get('Tags') or []) == (tags or [])]
         return (gateway or [None])[0]
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
@@ -1781,8 +2067,9 @@ class EC2SubnetRouteTableAssociation(GenericBaseModel):
 
     def fetch_state(self, stack_name, resources):
         client = aws_stack.connect_to_service('ec2')
-        table_id = self.resolve_refs_recursively(stack_name, self.props.get('RouteTableId'), resources)
-        gw_id = self.resolve_refs_recursively(stack_name, self.props.get('GatewayId'), resources)
+        props = self.props
+        table_id = self.resolve_refs_recursively(stack_name, props.get('RouteTableId'), resources)
+        gw_id = self.resolve_refs_recursively(stack_name, props.get('GatewayId'), resources)
         route_tables = client.describe_route_tables()['RouteTables']
         route_table = ([t for t in route_tables if t['RouteTableId'] == table_id] or [None])[0]
         if route_table:
@@ -1814,8 +2101,9 @@ class EC2VPCGatewayAttachment(GenericBaseModel):
 
     def fetch_state(self, stack_name, resources):
         client = aws_stack.connect_to_service('ec2')
-        igw_id = self.resolve_refs_recursively(stack_name, self.props.get('InternetGatewayId'), resources)
-        vpngw_id = self.resolve_refs_recursively(stack_name, self.props.get('VpnGatewayId'), resources)
+        props = self.props
+        igw_id = self.resolve_refs_recursively(stack_name, props.get('InternetGatewayId'), resources)
+        vpngw_id = self.resolve_refs_recursively(stack_name, props.get('VpnGatewayId'), resources)
         gateways = []
         if igw_id:
             gateways = client.describe_internet_gateways()['InternetGateways']
@@ -1823,20 +2111,29 @@ class EC2VPCGatewayAttachment(GenericBaseModel):
         elif vpngw_id:
             gateways = client.describe_vpn_gateways()['VpnGateways']
             gateways = [g for g in gateways if g['VpnGatewayId'] == vpngw_id]
-        return (gateways or [None])[0]
+        gateway = (gateways or [{}])[0]
+        attachments = gateway.get('Attachments') or gateway.get('VpcAttachments') or []
+        result = [a for a in attachments if a.get('State') in ('attached', 'available')]
+        if result:
+            return gateway
 
     def get_physical_resource_id(self, attribute=None, **kwargs):
-        return self.props.get('RouteTableAssociationId')
+        props = self.props
+        gw_id = props.get('VpnGatewayId') or props.get('InternetGatewayId')
+        attachment = (props.get('Attachments') or props.get('VpcAttachments') or [{}])[0]
+        if attachment:
+            result = '%s-%s' % (gw_id, attachment.get('VpcId'))
+            return result
 
-    @staticmethod
-    def get_deploy_templates():
+    @classmethod
+    def get_deploy_templates(cls):
         def _attach_gateway(resource_id, resources, *args, **kwargs):
             client = aws_stack.connect_to_service('ec2')
-            resource = resources[resource_id]
-            resource_props = resource.get('Properties')
-            igw_id = resource_props.get('InternetGatewayId')
-            vpngw_id = resource_props.get('VpnGatewayId')
-            vpc_id = resource_props.get('VpcId')
+            resource = cls(resources[resource_id])
+            props = resource.props
+            igw_id = props.get('InternetGatewayId')
+            vpngw_id = props.get('VpnGatewayId')
+            vpc_id = props.get('VpcId')
             if igw_id:
                 client.attach_internet_gateway(VpcId=vpc_id, InternetGatewayId=igw_id)
             elif vpngw_id:
