@@ -32,12 +32,16 @@ LOG = logging.getLogger(__name__)
 # this process is started as root, then we cannot kill it from a non-root process
 HEADER_KILL_SIGNAL = 'x-localstack-kill'
 
+# Header to indicate the current API (service) being called
+HEADER_TARGET_API = 'x-localstack-tgt-api'
+
 # lock obtained during boostrapping (persistence restoration) to avoid concurrency issues
 BOOTSTRAP_LOCK = threading.RLock()
 
 GZIP_ENCODING = 'GZIP'
 IDENTITY_ENCODING = 'IDENTITY'
 S3 = 's3'
+API_UNKNOWN = '_unknown_'
 
 
 class ProxyListenerEdge(ProxyListener):
@@ -83,7 +87,7 @@ class ProxyListenerEdge(ProxyListener):
                     LOG.debug('OUT(%s): "%s %s" - status: %s' % (api, method, path, 200))
                 return 200
 
-            if api in ['', None, '_unknown_']:
+            if api in ['', None, API_UNKNOWN]:
                 truncated = truncate(data)
                 if auth_header or target or data or path not in ['/', '/favicon.ico']:
                     LOG.info(('Unable to find forwarding rule for host "%s", path "%s %s", '
@@ -99,6 +103,7 @@ class ProxyListenerEdge(ProxyListener):
 
         if api and not headers.get('Authorization'):
             headers['Authorization'] = aws_stack.mock_aws_request_headers(api)['Authorization']
+        headers[HEADER_TARGET_API] = str(api)
 
         headers['Host'] = host
         if isinstance(data, dict):
@@ -117,15 +122,17 @@ class ProxyListenerEdge(ProxyListener):
             return do_forward_request(api, method, path, data, headers, port=port)
 
     def return_response(self, method, path, data, headers, response, request_handler=None):
+        api = headers.get(HEADER_TARGET_API) or ''
 
         if config.LS_LOG:
             # print response trace for debugging, if enabled
-            api, port, path, host = get_api_from_headers(headers, method=method, path=path, data=data)
-            if api and api != '_unknown_':
+            if api and api != API_UNKNOWN:
                 LOG.debug('OUT(%s): "%s %s" - status: %s - response headers: %s - response: %s' %
                     (api, method, path, response.status_code, dict(response.headers), response.content))
 
-        if headers.get('Accept-Encoding') == 'gzip' and response._content:
+        # Fix Go SDK issue
+        # https://github.com/localstack/localstack/issues/3833
+        if headers.get('Accept-Encoding') == 'gzip' and response._content and api not in [S3]:
             response._content = gzip.compress(to_bytes(response._content))
             response.headers['Content-Length'] = str(len(response._content))
             response.headers['Content-Encoding'] = 'gzip'
@@ -152,7 +159,7 @@ def do_forward_request_inmem(api, method, path, data, headers, port=None):
     # TODO determine client address..?
     client_address = LOCALHOST_IP
     server_address = headers.get('host') or LOCALHOST
-    forward_url = 'http://%s:%s' % (config.HOSTNAME, backend_port)
+    forward_url = 'http://%s:%s' % (LOCALHOST, backend_port)
     response = modify_and_forward(method=method, path=path, data_bytes=data, headers=headers,
         forward_base_url=forward_url, listeners=[listener], request_handler=None,
         client_address=client_address, server_address=server_address)
@@ -160,7 +167,8 @@ def do_forward_request_inmem(api, method, path, data, headers, port=None):
 
 
 def do_forward_request_network(port, method, path, data, headers):
-    connect_host = '%s:%s' % (config.HOSTNAME, port)
+    # TODO: enable per-service endpoints, to allow deploying in distributed settings
+    connect_host = '%s:%s' % (LOCALHOST, port)
     url = '%s://%s%s' % (get_service_protocol(), connect_host, path)
     function = getattr(requests, method.lower())
     response = function(url, data=data, headers=headers, verify=False, stream=True)
@@ -220,7 +228,7 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
     path = path or '/'
 
     # initialize result
-    result = '_unknown_', 0
+    result = API_UNKNOWN, 0
 
     # https://docs.aws.amazon.com/general/latest/gr/sigv4-signed-request-examples.html
     try:
@@ -248,6 +256,8 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
         result = 'route53', config.PORT_ROUTE53
     elif result[0] == 'monitoring':
         result = 'cloudwatch', config.PORT_CLOUDWATCH
+    elif result[0] == 'email':
+        result = 'ses', config.PORT_SES
     elif result[0] == 'execute-api' or '.execute-api.' in host:
         result = 'apigateway', config.PORT_APIGATEWAY
     elif target.startswith('Firehose_'):
@@ -261,6 +271,10 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
         result = 'web', config.PORT_WEB_UI
     elif result[0] == 'EventBridge' or target.startswith('AWSEvents'):
         result = 'events', config.PORT_EVENTS
+    elif target.startswith('ResourceGroupsTaggingAPI_'):
+        result = 'resourcegroupstaggingapi', config.PORT_RESOURCEGROUPSTAGGINGAPI
+    elif result[0] == 'resource-groups':
+        result = 'resource-groups', config.PORT_RESOURCE_GROUPS
 
     return result[0], result_before[1] or result[1], path, host
 
