@@ -40,6 +40,12 @@ URL_LOCALSTACK_FAT_JAR = ('https://repo1.maven.org/maven2/' +
     'cloud/localstack/localstack-utils/{v}/localstack-utils-{v}-fat.jar').format(v=LOCALSTACK_MAVEN_VERSION)
 MARKER_FILE_LIGHT_VERSION = '%s/.light-version' % INSTALL_DIR_INFRA
 IMAGE_NAME_SFN_LOCAL = 'amazon/aws-stepfunctions-local'
+ARTIFACTS_REPO = 'https://github.com/localstack/localstack-artifacts'
+SFN_PATCH_CLASS = 'com/amazonaws/stepfunctions/local/runtime/executors/task/LambdaTaskStateExecutor.class'
+SFN_PATCH_CLASS_URL = '%s/raw/master/stepfunctions-local-patch/%s' % (ARTIFACTS_REPO, SFN_PATCH_CLASS)
+
+DEBUGPY_MODULE='debugpy'
+DEBUGPY_DEPENDENCIES=['gcc', 'python3-dev', 'musl-dev']
 
 # Target version for javac, to ensure compatibility with earlier JREs
 JAVAC_TARGET_VERSION = '1.8'
@@ -47,8 +53,6 @@ JAVAC_TARGET_VERSION = '1.8'
 # SQS backend implementation provider - either "moto" or "elasticmq"
 SQS_BACKEND_IMPL = os.environ.get('SQS_PROVIDER') or 'moto'
 
-# As of 2019-10-09, the DDB fix (see below) doesn't seem to be required anymore
-APPLY_DDB_ALPINE_FIX = False
 # TODO: 2019-10-09: Temporarily overwriting DDB, as we're hitting a SIGSEGV JVM crash with the latest version
 OVERWRITE_DDB_FILES_IN_DOCKER = False
 
@@ -133,7 +137,7 @@ def install_elasticmq():
         log_install_msg('ElasticMQ')
         mkdir(INSTALL_DIR_ELASTICMQ)
         # download archive
-        tmp_archive = os.path.join(tempfile.gettempdir(), 'elasticmq-server.jar')
+        tmp_archive = os.path.join(config.TMP_FOLDER, 'elasticmq-server.jar')
         if not os.path.exists(tmp_archive):
             download(ELASTICMQ_JAR_URL, tmp_archive)
         shutil.copy(tmp_archive, INSTALL_DIR_ELASTICMQ)
@@ -170,6 +174,14 @@ def install_stepfunctions_local():
             dn=docker_name, tgt=INSTALL_DIR_INFRA))
         run('mv %s/stepfunctionslocal/*.jar %s' % (INSTALL_DIR_INFRA, INSTALL_DIR_STEPFUNCTIONS))
         rm_rf('%s/stepfunctionslocal' % INSTALL_DIR_INFRA)
+    # apply patches
+    patch_class_file = os.path.join(INSTALL_DIR_STEPFUNCTIONS, SFN_PATCH_CLASS)
+    if not os.path.exists(patch_class_file):
+        download(SFN_PATCH_CLASS_URL, patch_class_file)
+        cmd = 'cd "%s"; zip %s %s' % (INSTALL_DIR_STEPFUNCTIONS, INSTALL_PATH_STEPFUNCTIONS_JAR, SFN_PATCH_CLASS)
+        run(cmd)
+
+
 
 
 def install_dynamodb_local():
@@ -183,20 +195,6 @@ def install_dynamodb_local():
         dynamodb_url = DYNAMODB_JAR_URL_ALPINE if is_in_alpine else DYNAMODB_JAR_URL
         download_and_extract_with_retry(dynamodb_url, tmp_archive, INSTALL_DIR_DDB)
 
-    # fix for Alpine, otherwise DynamoDBLocal fails with:
-    # DynamoDBLocal_lib/libsqlite4java-linux-amd64.so: __memcpy_chk: symbol not found
-    if is_in_alpine:
-        ddb_libs_dir = '%s/DynamoDBLocal_lib' % INSTALL_DIR_DDB
-        patched_marker = '%s/alpine_fix_applied' % ddb_libs_dir
-        if APPLY_DDB_ALPINE_FIX and not os.path.exists(patched_marker):
-            patched_lib = ('https://rawgit.com/bhuisgen/docker-alpine/master/alpine-dynamodb/' +
-                'rootfs/usr/local/dynamodb/DynamoDBLocal_lib/libsqlite4java-linux-amd64.so')
-            patched_jar = ('https://rawgit.com/bhuisgen/docker-alpine/master/alpine-dynamodb/' +
-                'rootfs/usr/local/dynamodb/DynamoDBLocal_lib/sqlite4java.jar')
-            run("curl -L -o %s/libsqlite4java-linux-amd64.so '%s'" % (ddb_libs_dir, patched_lib))
-            run("curl -L -o %s/sqlite4java.jar '%s'" % (ddb_libs_dir, patched_jar))
-            save_file(patched_marker, '')
-
     # fix logging configuration for DynamoDBLocal
     log4j2_config = """<Configuration status="WARN">
       <Appenders>
@@ -273,6 +271,14 @@ def install_all_components():
     # install all components
     install_components(DEFAULT_SERVICE_PORTS.keys())
 
+def install_debugpy_and_dependencies():
+    try:
+        import debugpy
+    except ModuleNotFoundError:
+        run('apk fetch %s --output %s'%  (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER))
+        run('apk add %s --cache-dir %s' %  (' '.join(DEBUGPY_DEPENDENCIES), config.TMP_FOLDER ) )
+        run('pip install %s' % DEBUGPY_MODULE )
+
 
 # -----------------
 # HELPER FUNCTIONS
