@@ -10,6 +10,7 @@ from localstack.utils.aws import aws_stack
 from localstack.services.s3 import s3_listener, s3_utils
 from localstack.utils.server import multiserver
 from localstack.utils.common import wait_for_port_open, get_free_tcp_port
+from localstack.utils.generic.dict_utils import get_safe
 from localstack.services.infra import start_moto_server
 from localstack.services.awslambda.lambda_api import BUCKET_MARKER_LOCAL
 from urllib.parse import urlparse
@@ -21,6 +22,7 @@ S3_MAX_FILE_SIZE_MB = 2048
 
 # temporary state
 TMP_STATE = {}
+TMP_TAG = {}
 
 # Key for tracking patch applience
 PATCHES_APPLIED = 'S3_PATCHED'
@@ -130,6 +132,17 @@ def apply_patches():
     def s3_key_response_post(self, request, body, bucket_name, query, key_name, *args, **kwargs):
         result = s3_key_response_post_orig(request, body, bucket_name, query, key_name, *args, **kwargs)
         s3_update_acls(self, request, query, bucket_name, key_name)
+        try:
+            if query.get('uploadId'):
+                if (bucket_name, key_name) in TMP_TAG:
+                    key = self.backend.get_object(bucket_name, key_name)
+                    self.backend.set_key_tags(key, TMP_TAG.get((bucket_name, key_name), None), key_name)
+                    TMP_TAG.pop((bucket_name, key_name))
+        except Exception:
+            pass
+        if query.get('uploads') and request.headers.get('X-Amz-Tagging'):
+            tags = self._tagging_from_headers(request.headers)
+            TMP_TAG[(bucket_name, key_name)] = tags
         return result
 
     s3_key_response_post_orig = s3_responses.S3ResponseInstance._key_response_post
@@ -278,6 +291,7 @@ def apply_patches():
     # Patch utils_is_delete_keys
     # https://github.com/localstack/localstack/issues/2866
     # https://github.com/localstack/localstack/issues/2850
+    # https://github.com/localstack/localstack/issues/3931
     utils_is_delete_keys_orig = s3bucket_path_utils.is_delete_keys
 
     def utils_is_delete_keys(request, path, bucket_name):
@@ -285,7 +299,10 @@ def apply_patches():
 
     def s3_response_is_delete_keys(self, request, path, bucket_name):
         if self.subdomain_based_buckets(request):
-            return is_delete_keys(request, path, bucket_name)
+            # Temporary fix until moto supports x-id and DeleteObjects (#3931)
+            query = self._get_querystring(request.url)
+            is_delete_keys_v3 = (query and ('delete' in query) and get_safe(query, '$.x-id.0') == 'DeleteObjects')
+            return is_delete_keys_v3 or is_delete_keys(request, path, bucket_name)
         else:
             return utils_is_delete_keys(request, path, bucket_name)
 
@@ -332,3 +349,16 @@ def apply_patches():
 
     s3_responses.S3ResponseInstance._bucket_response_get = types.MethodType(
         s3_bucket_response_get, s3_responses.S3ResponseInstance)
+
+    copy_key_orig = s3_models.s3_backend.copy_key
+
+    def copy_key(
+        self, src_bucket_name, src_key_name, dest_bucket_name, dest_key_name,
+        storage=None, acl=None, src_version_id=None, *args, **kwargs
+    ):
+        copy_key_orig(src_bucket_name, src_key_name, dest_bucket_name,
+            dest_key_name, storage=storage, acl=acl, src_version_id=src_version_id, *args, **kwargs)
+        key = self.get_object(dest_bucket_name, dest_key_name)
+        key._etag = None
+
+    s3_models.s3_backend.copy_key = types.MethodType(copy_key, s3_models.s3_backend)
