@@ -16,31 +16,35 @@ from localstack.services.awslambda import lambda_api
 from localstack.services.install import SQS_BACKEND_IMPL
 from localstack.utils.analytics import event_publisher
 from localstack.utils.aws import aws_stack
-from localstack.utils.aws.aws_responses import response_regex_replace
+from localstack.utils.aws.aws_responses import response_regex_replace, create_sqs_system_attributes
 from localstack.utils.aws.dead_letter_queue import sns_error_to_dead_letter_queue
 from localstack.utils.common import parse_request_data, timestamp_millis, short_uid, to_str, to_bytes, start_thread
 from localstack.utils.persistence import PersistingProxyListener
+from localstack.services.generic_proxy import RegionBackend
+from moto.sns.models import SNSBackend as MotoSNSBackend
+from moto.sns.exceptions import DuplicateSnsEndpointError
 
 # set up logger
 LOG = logging.getLogger(__name__)
 
-# mappings for SNS topic subscriptions
-SNS_SUBSCRIPTIONS = {}
+# additional attributes used for HTTP subscriptions
+HTTP_SUBSCRIPTION_ATTRIBUTES = ['UnsubscribeURL']
 
-# mappings for subscription status
-SUBSCRIPTION_STATUS = {}
 
-# mappings for SNS tags
-SNS_TAGS = {}
-
-# cache of platform endpoint messages (used primarily for testing)
-PLATFORM_ENDPOINT_MESSAGES = {}
-
-# maps phone numbers to list of sent messages
-SMS_MESSAGES = []
-
-# actions to be skipped from persistence
-SKIP_PERSISTENCE_ACTIONS = ['Subscribe', 'ConfirmSubscription', 'Unsubscribe']
+class SNSBackend(RegionBackend):
+    def __init__(self):
+        # maps topic ARN to list of subscriptions
+        self.sns_subscriptions = {}
+        # maps subscription ARN to subscription status
+        self.subscription_status = {}
+        # maps topic ARN to list of tags
+        self.sns_tags = {}
+        # cache of topic ARN to platform endpoint messages (used primarily for testing)
+        self.platform_endpoint_messages = {}
+        # maps phone numbers to list of sent messages
+        self.sms_messages = []
+        # actions to be skipped from persistence
+        self.skip_persistence_actions = ['Subscribe', 'ConfirmSubscription', 'Unsubscribe']
 
 
 class ProxyListenerSNS(PersistingProxyListener):
@@ -69,11 +73,9 @@ class ProxyListenerSNS(PersistingProxyListener):
 
             req_action = req_data['Action'][0]
             topic_arn = req_data.get('TargetArn') or req_data.get('TopicArn') or req_data.get('ResourceArn')
-
             if topic_arn:
                 topic_arn = topic_arn[0]
                 topic_arn = aws_stack.fix_account_id_in_arns(topic_arn)
-
             if req_action == 'SetSubscriptionAttributes':
                 sub = get_subscription_by_arn(req_data['SubscriptionArn'][0])
                 if not sub:
@@ -87,10 +89,14 @@ class ProxyListenerSNS(PersistingProxyListener):
             elif req_action == 'GetSubscriptionAttributes':
                 sub = get_subscription_by_arn(req_data['SubscriptionArn'][0])
                 if not sub:
-                    return make_error(message='Unable to find subscription for given ARN', code=400)
+                    return make_error(message='Subscription with arn {0} not found'.
+                                      format(req_data['SubscriptionArn'][0]),
+                                      code=404, code_string='NotFound')
 
                 content = '<Attributes>'
                 for key, value in sub.items():
+                    if key in HTTP_SUBSCRIPTION_ATTRIBUTES:
+                        continue
                     content += '<entry><key>%s</key><value>%s</value></entry>\n' % (key, value)
                 content += '</Attributes>'
                 return make_response(req_action, content=content)
@@ -120,11 +126,11 @@ class ProxyListenerSNS(PersistingProxyListener):
             elif req_action == 'Publish':
                 if req_data.get('Subject') == ['']:
                     return make_error(code=400, code_string='InvalidParameter', message='Subject')
-
+                sns_backend = SNSBackend.get()
                 # No need to create a topic to send SMS or single push notifications with SNS
                 # but we can't mock a sending so we only return that it went well
                 if 'PhoneNumber' not in req_data and 'TargetArn' not in req_data:
-                    if topic_arn not in SNS_SUBSCRIPTIONS:
+                    if topic_arn not in sns_backend.sns_subscriptions:
                         return make_error(code=404, code_string='NotFound', message='Topic does not exist')
 
                 message_id = publish_message(topic_arn, req_data, headers)
@@ -146,16 +152,18 @@ class ProxyListenerSNS(PersistingProxyListener):
                 return make_response(req_action, content=content)
 
             elif req_action == 'CreateTopic':
+                sns_backend = SNSBackend.get()
                 topic_arn = aws_stack.sns_topic_arn(req_data['Name'][0])
-                tag_resource_success = self._extract_tags(topic_arn, req_data, True)
-                SNS_SUBSCRIPTIONS[topic_arn] = SNS_SUBSCRIPTIONS.get(topic_arn) or []
+                tag_resource_success = self._extract_tags(topic_arn, req_data, True, sns_backend)
+                sns_backend.sns_subscriptions[topic_arn] = sns_backend.sns_subscriptions.get(topic_arn) or []
                 # in case if there is an error it returns an error , other wise it will continue as expected.
                 if not tag_resource_success:
                     return make_error(code=400, code_string='InvalidParameter',
                                   message='Topic already exists with different tags')
 
             elif req_action == 'TagResource':
-                self._extract_tags(topic_arn, req_data, False)
+                sns_backend = SNSBackend.get()
+                self._extract_tags(topic_arn, req_data, False, sns_backend)
                 return make_response(req_action)
 
             elif req_action == 'UntagResource':
@@ -173,21 +181,20 @@ class ProxyListenerSNS(PersistingProxyListener):
         return True
 
     @staticmethod
-    def _extract_tags(topic_arn, req_data, is_create_topic_request):
+    def _extract_tags(topic_arn, req_data, is_create_topic_request, sns_backend):
         tags = []
         req_tags = {k: v for k, v in req_data.items() if k.startswith('Tags.member.')}
+        existing_tags = sns_backend.sns_tags.get(topic_arn, None)
         # TODO: use aws_responses.extract_tags(...) here!
         for i in range(int(len(req_tags.keys()) / 2)):
             key = req_tags['Tags.member.' + str(i + 1) + '.Key'][0]
             value = req_tags['Tags.member.' + str(i + 1) + '.Value'][0]
-            tags.append({'Key': key, 'Value': value})
-
+            tag = {'Key': key, 'Value': value}
+            tags.append(tag)
             # this means topic already created with empty tags and when we try to create it
             # again with other tag value then it should fail according to aws documentation.
-            existing_tags = SNS_TAGS.get(topic_arn, None)
-            if is_create_topic_request and existing_tags is not None and existing_tags != tags:
+            if is_create_topic_request and existing_tags is not None and tag not in existing_tags:
                 return False
-
         do_tag_resource(topic_arn, tags)
         return True
 
@@ -248,21 +255,40 @@ class ProxyListenerSNS(PersistingProxyListener):
                 )
 
     def should_persist(self, method, path, data, headers, response):
+        sns_backend = SNSBackend.get()
         req_params = parse_request_data(method, path, data)
         action = req_params.get('Action', '')
-        if action in SKIP_PERSISTENCE_ACTIONS:
+        if action in sns_backend.skip_persistence_actions:
             return False
         return super(ProxyListenerSNS, self).should_persist(method, path, data, headers, response)
 
 
+def patch_moto():
+    def patch_create_platform_endpoint(self, *args):
+        try:
+            return create_platform_endpoint_orig(self, *args)
+        except DuplicateSnsEndpointError:
+            custom_user_data, token = args[2], args[3]
+            for endpoint in self.platform_endpoints.values():
+                if endpoint.token == token:
+                    if custom_user_data and custom_user_data != endpoint.custom_user_data:
+                        raise DuplicateSnsEndpointError('Endpoint already exist for token: %s with different attributes'
+                             % token)
+                    return endpoint
+    create_platform_endpoint_orig = MotoSNSBackend.create_platform_endpoint
+    MotoSNSBackend.create_platform_endpoint = patch_create_platform_endpoint
+
+
+patch_moto()
 # instantiate listener
 UPDATE_SNS = ProxyListenerSNS()
 
 
 def unsubscribe_sqs_queue(queue_url):
     """ Called upon deletion of an SQS queue, to remove the queue from subscriptions """
-    for topic_arn, subscriptions in SNS_SUBSCRIPTIONS.items():
-        subscriptions = SNS_SUBSCRIPTIONS.get(topic_arn, [])
+    sns_backend = SNSBackend.get()
+    for topic_arn, subscriptions in sns_backend.sns_subscriptions.items():
+        subscriptions = sns_backend.sns_subscriptions.get(topic_arn, [])
         for subscriber in list(subscriptions):
             sub_url = subscriber.get('sqs_queue_url') or subscriber['Endpoint']
             if queue_url == sub_url:
@@ -270,8 +296,8 @@ def unsubscribe_sqs_queue(queue_url):
 
 
 def message_to_subscribers(message_id, message, topic_arn, req_data, headers, subscription_arn=None, skip_checks=False):
-
-    subscriptions = SNS_SUBSCRIPTIONS.get(topic_arn, [])
+    sns_backend = SNSBackend.get()
+    subscriptions = sns_backend.sns_subscriptions.get(topic_arn, [])
     for subscriber in list(subscriptions):
         if subscription_arn not in [None, subscriber['SubscriptionArn']]:
             continue
@@ -288,13 +314,15 @@ def message_to_subscribers(message_id, message, topic_arn, req_data, headers, su
                 'endpoint': subscriber['Endpoint'],
                 'message_content': req_data['Message'][0]
             }
-            SMS_MESSAGES.append(event)
+            sns_backend.sms_messages.append(event)
             LOG.info('Delivering SMS message to %s: %s', subscriber['Endpoint'], req_data['Message'][0])
 
         elif subscriber['Protocol'] == 'sqs':
             queue_url = None
+
             try:
                 endpoint = subscriber['Endpoint']
+
                 if 'sqs_queue_url' in subscriber:
                     queue_url = subscriber.get('sqs_queue_url')
                 elif '://' in endpoint:
@@ -397,12 +425,14 @@ def message_to_subscribers(message_id, message, topic_arn, req_data, headers, su
 
 
 def publish_message(topic_arn, req_data, headers, subscription_arn=None, skip_checks=False):
+    sns_backend = SNSBackend.get()
     message = req_data['Message'][0]
     message_id = str(uuid.uuid4())
 
     if topic_arn and ':endpoint/' in topic_arn:
         # cache messages published to platform endpoints
-        cache = PLATFORM_ENDPOINT_MESSAGES[topic_arn] = PLATFORM_ENDPOINT_MESSAGES.get(topic_arn) or []
+        cache = sns_backend.platform_endpoint_messages[topic_arn] = sns_backend. \
+            platform_endpoint_messages.get(topic_arn) or []
         cache.append(req_data)
 
     LOG.debug('Publishing message to TopicArn: %s | Message: %s' % (topic_arn, message))
@@ -413,19 +443,21 @@ def publish_message(topic_arn, req_data, headers, subscription_arn=None, skip_ch
 
 
 def do_delete_topic(topic_arn):
-    SNS_SUBSCRIPTIONS.pop(topic_arn, None)
-    SNS_TAGS.pop(topic_arn, None)
+    sns_backend = SNSBackend.get()
+    sns_backend.sns_subscriptions.pop(topic_arn, None)
+    sns_backend.sns_tags.pop(topic_arn, None)
 
 
 def do_confirm_subscription(topic_arn, token):
-    for k, v in SUBSCRIPTION_STATUS.items():
+    sns_backend = SNSBackend.get()
+    for k, v in sns_backend.subscription_status.items():
         if v['Token'] == token and v['TopicArn'] == topic_arn:
             v['Status'] = 'Subscribed'
 
 
 def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, filter_policy=None):
-    topic_subs = SNS_SUBSCRIPTIONS[topic_arn] = SNS_SUBSCRIPTIONS.get(topic_arn) or []
-
+    sns_backend = SNSBackend.get()
+    topic_subs = sns_backend.sns_subscriptions[topic_arn] = sns_backend.sns_subscriptions.get(topic_arn) or []
     # An endpoint may only be subscribed to a topic once. Subsequent
     # subscribe calls do nothing (subscribe is idempotent).
     for existing_topic_subscription in topic_subs:
@@ -443,10 +475,10 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
     subscription.update(attributes)
     topic_subs.append(subscription)
 
-    if subscription_arn not in SUBSCRIPTION_STATUS:
-        SUBSCRIPTION_STATUS[subscription_arn] = {}
+    if subscription_arn not in sns_backend.subscription_status:
+        sns_backend.subscription_status[subscription_arn] = {}
 
-    SUBSCRIPTION_STATUS[subscription_arn].update(
+    sns_backend.subscription_status[subscription_arn].update(
         {
             'TopicArn': topic_arn,
             'Token': short_uid(),
@@ -457,6 +489,7 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
     if protocol in ['http', 'https']:
         token = short_uid()
         external_url = external_service_url('sns')
+        subscription['UnsubscribeURL'] = '%s/?Action=Unsubscribe&SubscriptionArn=%s' % (external_url, subscription_arn)
         confirmation = {
             'Type': ['SubscriptionConfirmation'],
             'Token': [token],
@@ -468,18 +501,20 @@ def do_subscribe(topic_arn, endpoint, protocol, subscription_arn, attributes, fi
 
 
 def do_unsubscribe(subscription_arn):
-    for topic_arn, existing_subs in SNS_SUBSCRIPTIONS.items():
-        SNS_SUBSCRIPTIONS[topic_arn] = [
+    sns_backend = SNSBackend.get()
+    for topic_arn, existing_subs in sns_backend.sns_subscriptions.items():
+        sns_backend.sns_subscriptions[topic_arn] = [
             sub for sub in existing_subs
             if sub['SubscriptionArn'] != subscription_arn
         ]
 
 
 def _get_tags(topic_arn):
-    if topic_arn not in SNS_TAGS:
-        SNS_TAGS[topic_arn] = []
+    sns_backend = SNSBackend.get()
+    if topic_arn not in sns_backend.sns_tags:
+        sns_backend.sns_tags[topic_arn] = []
 
-    return SNS_TAGS[topic_arn]
+    return sns_backend.sns_tags[topic_arn]
 
 
 def do_list_tags_for_resource(topic_arn):
@@ -487,7 +522,8 @@ def do_list_tags_for_resource(topic_arn):
 
 
 def do_tag_resource(topic_arn, tags):
-    existing_tags = SNS_TAGS.get(topic_arn, [])
+    sns_backend = SNSBackend.get()
+    existing_tags = sns_backend.sns_tags.get(topic_arn, [])
     tags = [
         tag for idx, tag in enumerate(tags)
         if tag not in tags[:idx]
@@ -506,11 +542,12 @@ def do_tag_resource(topic_arn, tags):
         else:
             existing_tags[existing_index] = item
 
-    SNS_TAGS[topic_arn] = existing_tags
+    sns_backend.sns_tags[topic_arn] = existing_tags
 
 
 def do_untag_resource(topic_arn, tag_keys):
-    SNS_TAGS[topic_arn] = [t for t in _get_tags(topic_arn) if t['Key'] not in tag_keys]
+    sns_backend = SNSBackend.get()
+    sns_backend.sns_tags[topic_arn] = [t for t in _get_tags(topic_arn) if t['Key'] not in tag_keys]
 
 
 # ---------------
@@ -519,8 +556,9 @@ def do_untag_resource(topic_arn, tag_keys):
 
 
 def get_subscription_by_arn(sub_arn):
+    sns_backend = SNSBackend.get()
     # TODO maintain separate map instead of traversing all items
-    for key, subscriptions in SNS_SUBSCRIPTIONS.items():
+    for key, subscriptions in sns_backend.sns_subscriptions.items():
         for sub in subscriptions:
             if sub['SubscriptionArn'] == sub_arn:
                 return sub
@@ -590,6 +628,10 @@ def create_sns_message_body(subscriber, req_data, message_id=None):
         if req_data.get(key):
             data[key] = req_data[key][0]
 
+    for key in HTTP_SUBSCRIPTION_ATTRIBUTES:
+        if key in subscriber:
+            data[key] = subscriber[key]
+
     attributes = get_message_attributes(req_data)
     if attributes:
         data['MessageAttributes'] = attributes
@@ -598,8 +640,9 @@ def create_sns_message_body(subscriber, req_data, message_id=None):
 
 
 def create_sqs_message_attributes(subscriber, attributes):
-    if not is_raw_message_delivery(subscriber):
-        return {}
+    # Message attributes are going empty because of this hence removing this check.
+    # if not is_raw_message_delivery(subscriber):
+    #     return {}
 
     message_attributes = {}
     for key, value in attributes.items():
@@ -616,17 +659,6 @@ def create_sqs_message_attributes(subscriber, attributes):
     return message_attributes
 
 
-def create_sqs_system_attributes(headers):
-
-    system_attributes = {}
-    if 'X-Amzn-Trace-Id' in headers:
-        system_attributes['AWSTraceHeader'] = {
-            'DataType': 'String',
-            'StringValue': str(headers['X-Amzn-Trace-Id'])
-        }
-    return system_attributes
-
-
 def get_message_attributes(req_data):
     attributes = {}
     x = 1
