@@ -12,18 +12,23 @@ MAX_HEAP_SIZE = '256m'
 
 
 def get_command(backend_port):
-    dynamodb_endpoint = aws_stack.get_local_service_url('dynamodb')
-    sns_endpoint = aws_stack.get_local_service_url('sns')
-    sqs_endpoint = aws_stack.get_local_service_url('sqs')
-    sfn_endpoint = aws_stack.get_local_service_url('stepfunctions')
     cmd = ('cd %s; PORT=%s java -Dcom.amazonaws.sdk.disableCertChecking -Xmx%s -jar StepFunctionsLocal.jar '
-           '--dynamodb-endpoint %s --sns-endpoint %s '
-           '--sqs-endpoint %s --aws-region %s --aws-account %s --step-functions-endpoint %s') % (
-        install.INSTALL_DIR_STEPFUNCTIONS, backend_port, MAX_HEAP_SIZE, dynamodb_endpoint,
-        sns_endpoint, sqs_endpoint, aws_stack.get_region(), TEST_AWS_ACCOUNT_ID, sfn_endpoint)
+           '--aws-region %s --aws-account %s') % (
+        install.INSTALL_DIR_STEPFUNCTIONS, backend_port, MAX_HEAP_SIZE,
+        aws_stack.get_region(), TEST_AWS_ACCOUNT_ID)
     if config.STEPFUNCTIONS_LAMBDA_ENDPOINT.lower() != 'default':
         lambda_endpoint = config.STEPFUNCTIONS_LAMBDA_ENDPOINT or aws_stack.get_local_service_url('lambda')
         cmd += (' --lambda-endpoint %s') % (lambda_endpoint)
+    # add service endpoint flags
+    services = ['athena', 'batch', 'dynamodb', 'ecs', 'eks', 'glue', 'sagemaker', 'sns', 'sqs', 'stepfunctions']
+    for service in services:
+        flag = '--%s-endpoint' % service
+        if service == 'stepfunctions':
+            flag = '--step-functions-endpoint'
+        elif service in ['athena', 'eks']:
+            flag = '--step-functions-%s' % service
+        endpoint = aws_stack.get_local_service_url(service)
+        cmd += ' %s %s' % (flag, endpoint)
     return cmd
 
 
