@@ -91,8 +91,7 @@ def requests_error_response_xml_signature_calculation(message, string_to_sign=No
         parsed_response['Error']['StringToSign'] = string_to_sign
         parsed_response['Error']['SignatureProvided'] = signature
         parsed_response['Error']['StringToSignBytes'] = '{}'.format(bytes_signature.decode('utf-8'))
-        response._content = xmltodict.unparse(parsed_response)
-        response.headers['Content-Length'] = str(len(response._content))
+        set_response_content(response, xmltodict.unparse(parsed_response))
 
     if expires and code_string == 'AccessDenied':
 
@@ -101,13 +100,11 @@ def requests_error_response_xml_signature_calculation(message, string_to_sign=No
         parsed_response['Error']['Code'] = code_string
         parsed_response['Error']['Expires'] = '{}Z'.format(expires_isoformat)
         parsed_response['Error']['ServerTime'] = '{}Z'.format(server_time)
-        response._content = xmltodict.unparse(parsed_response)
-        response.headers['Content-Length'] = str(len(response._content))
+        set_response_content(response, xmltodict.unparse(parsed_response))
 
     if not signature and not expires and code_string == 'AccessDenied':
 
-        response._content = xmltodict.unparse(parsed_response)
-        response.headers['Content-Length'] = str(len(response._content))
+        set_response_content(response, xmltodict.unparse(parsed_response))
 
     if response._content:
         return response
@@ -145,6 +142,11 @@ def requests_to_flask_response(r):
     return Response(r.content, status=r.status_code, headers=dict(r.headers))
 
 
+def flask_not_found_error(msg=None):
+    msg = msg or 'The specified resource doesnt exist.'
+    return flask_error_response_json(msg, code=404, error_type='ResourceNotFoundException')
+
+
 def response_regex_replace(response, search, replace):
     content = re.sub(search, replace, to_str(response._content), flags=re.DOTALL | re.MULTILINE)
     set_response_content(response, content)
@@ -168,6 +170,17 @@ def make_error(*args, **kwargs):
     return flask_error_response_xml(*args, **kwargs)
 
 
+def create_sqs_system_attributes(headers):
+
+    system_attributes = {}
+    if 'X-Amzn-Trace-Id' in headers:
+        system_attributes['AWSTraceHeader'] = {
+            'DataType': 'String',
+            'StringValue': str(headers['X-Amzn-Trace-Id'])
+        }
+    return system_attributes
+
+
 def extract_tags(req_data):
     keys = []
     values = []
