@@ -3,6 +3,7 @@ import re
 import json
 import time
 import boto3
+import socket
 import logging
 import six
 import botocore
@@ -12,6 +13,7 @@ from localstack.constants import (
     APPLICATION_AMZ_JSON_1_0, APPLICATION_X_WWW_FORM_URLENCODED, TEST_AWS_ACCOUNT_ID,
     MAX_POOL_CONNECTIONS, TEST_AWS_ACCESS_KEY_ID, TEST_AWS_SECRET_ACCESS_KEY, S3_VIRTUAL_HOSTNAME)
 from localstack.utils.aws import templating
+from localstack.utils.generic import dict_utils
 from localstack.utils.common import (
     run_safe, to_str, is_string, is_string_or_bytes, make_http_request,
     is_port_open, get_service_protocol, retry, to_bytes)
@@ -47,6 +49,12 @@ DEFAULT_TIMER_LOOP_SECONDS = 60 * 50
 # maps SQS queue ARNs to queue URLs
 SQS_ARN_TO_URL_CACHE = {}
 
+# List of parameters with additional event target parameters
+EVENT_TARGET_PARAMETERS = ['$.SqsParameters', '$.KinesisParameters']
+
+# cached value used to determine the DNS status of the S3 hostname (whether it can be resolved properly)
+CACHE_S3_HOSTNAME_DNS_STATUS = None
+
 
 class Environment(object):
     def __init__(self, region=None, prefix=None):
@@ -186,6 +194,14 @@ def is_internal_call_context(headers):
 def set_internal_auth(headers):
     authorization = headers.get('Authorization') or ''
     authorization = re.sub(r'Credential=[^/]+/', 'Credential=%s/' % INTERNAL_AWS_ACCESS_KEY_ID, authorization)
+    if authorization.startswith('AWS '):
+        authorization = re.sub(r'AWS [^/]+',  # Cover Non HMAC Authentication
+                               'Credential=%s' % INTERNAL_AWS_ACCESS_KEY_ID,
+                               authorization)
+    else:
+        authorization = re.sub(r'Credential=[^/]+/',
+                               'Credential=%s/' % INTERNAL_AWS_ACCESS_KEY_ID,
+                               authorization)
     headers['Authorization'] = authorization
     return headers
 
@@ -243,10 +259,10 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
             if backend_url:
                 endpoint_url = backend_url
         config = config or botocore.client.Config()
-        # configure S3 path style addressing
+        # configure S3 path/host style addressing
         if service_name == 's3':
             if re.match(r'https?://localhost(:[0-9]+)?', endpoint_url):
-                endpoint_url = endpoint_url.replace('://localhost', '://%s' % S3_VIRTUAL_HOSTNAME)
+                endpoint_url = endpoint_url.replace('://localhost', '://%s' % get_s3_hostname())
         # To, prevent error "Connection pool is full, discarding connection ...",
         # set the environment variable MAX_POOL_CONNECTIONS. Default is 150.
         config.max_pool_connections = MAX_POOL_CONNECTIONS
@@ -259,6 +275,19 @@ def connect_to_service(service_name, client=True, env=None, region_name=None, en
     return BOTO_CLIENTS_CACHE[cache_key]
 
 
+def get_s3_hostname():
+    global CACHE_S3_HOSTNAME_DNS_STATUS
+    if CACHE_S3_HOSTNAME_DNS_STATUS is None:
+        try:
+            assert socket.gethostbyname(S3_VIRTUAL_HOSTNAME)
+            CACHE_S3_HOSTNAME_DNS_STATUS = True
+        except socket.error:
+            CACHE_S3_HOSTNAME_DNS_STATUS = False
+    if CACHE_S3_HOSTNAME_DNS_STATUS:
+        return S3_VIRTUAL_HOSTNAME
+    return LOCALHOST
+
+
 # TODO remove from here in the future
 def render_velocity_template(*args, **kwargs):
     return templating.render_velocity_template(*args, **kwargs)
@@ -267,11 +296,12 @@ def render_velocity_template(*args, **kwargs):
 def generate_presigned_url(*args, **kwargs):
     id_before = os.environ.get(ENV_ACCESS_KEY)
     key_before = os.environ.get(ENV_SECRET_KEY)
+    endpoint_url = kwargs.pop('endpoint_url', None)
     try:
         # Note: presigned URL needs to be created with test credentials
         os.environ[ENV_ACCESS_KEY] = TEST_AWS_ACCESS_KEY_ID
         os.environ[ENV_SECRET_KEY] = TEST_AWS_SECRET_ACCESS_KEY
-        s3_client = connect_to_service('s3', cache=False)
+        s3_client = connect_to_service('s3', endpoint_url=endpoint_url, cache=False)
         return s3_client.generate_presigned_url(*args, **kwargs)
     finally:
         if id_before:
@@ -556,6 +586,8 @@ def _resource_arn(name, pattern, account_id=None, region_name=None):
         return name
     account_id = get_account_id(account_id)
     region_name = region_name or get_region()
+    if len(pattern.split('%s')) == 3:
+        return pattern % (account_id, name)
     return pattern % (region_name, account_id, name)
 
 
@@ -573,7 +605,7 @@ def send_event_to_target(arn, event, target_attributes=None, asynchronous=True):
     elif ':sqs:' in arn:
         sqs_client = connect_to_service('sqs', region_name=region)
         queue_url = get_sqs_queue_url(arn)
-        msg_group_id = (target_attributes or {}).get('MessageGroupId')
+        msg_group_id = dict_utils.get_safe(target_attributes, '$.SqsParameters.MessageGroupId')
         kwargs = {'MessageGroupId': msg_group_id} if msg_group_id else {}
         sqs_client.send_message(QueueUrl=queue_url, MessageBody=json.dumps(event), **kwargs)
 
@@ -600,13 +632,29 @@ def send_event_to_target(arn, event, target_attributes=None, asynchronous=True):
             }]
         )
 
+    elif ':kinesis:' in arn:
+        partition_key_path = dict_utils.get_safe(
+            target_attributes,
+            '$.KinesisParameters.PartitionKeyPath',
+            default_value='$.id'
+        )
+
+        stream_name = arn.split('/')[-1]
+        partition_key = dict_utils.get_safe(event, partition_key_path, event['id'])
+        kinesis_client = connect_to_service('kinesis', region_name=region)
+
+        kinesis_client.put_record(
+            StreamName=stream_name,
+            Data=to_bytes(json.dumps(event)),
+            PartitionKey=partition_key
+        )
+
     else:
         LOG.warning('Unsupported Events rule target ARN: "%s"' % arn)
 
 
 def get_events_target_attributes(target):
-    # TODO: add support for other target types
-    return target.get('SqsParameters')
+    return dict_utils.pick_attributes(target, EVENT_TARGET_PARAMETERS)
 
 
 def get_or_create_bucket(bucket_name):
