@@ -1,5 +1,4 @@
 import re
-import os
 import json
 import base64
 import logging
@@ -15,13 +14,13 @@ from localstack.constants import TEST_AWS_ACCOUNT_ID, FALSE_STRINGS
 from localstack.services.s3 import s3_listener
 from localstack.utils.common import (
     json_safe, md5, canonical_json, short_uid, to_str, to_bytes,
-    mkdir, cp_r, prevent_stack_overflow, start_worker_thread, get_all_subclasses)
-from localstack.utils.testutil import create_zip_file, delete_all_s3_objects
+    prevent_stack_overflow, start_worker_thread, get_all_subclasses)
+from localstack.utils.testutil import delete_all_s3_objects
 from localstack.utils.cloudformation import template_preparer
-from localstack.services.awslambda.lambda_api import get_handler_file_from_name
-from localstack.services.cloudformation.service_models import GenericBaseModel, DependencyNotYetSatisfied
+from localstack.services.cloudformation.service_models import (
+    GenericBaseModel, DependencyNotYetSatisfied, KEY_RESOURCE_STATE)
 from localstack.services.cloudformation.deployment_utils import (
-    dump_json_params, select_parameters, param_defaults, remove_none_values, get_cfn_response_mod_file,
+    dump_json_params, select_parameters, param_defaults, remove_none_values,
     lambda_keys_to_lower, PLACEHOLDER_AWS_NO_VALUE, PLACEHOLDER_RESOURCE_NAME)
 
 ACTION_CREATE = 'create'
@@ -62,60 +61,12 @@ def rename_params(func, rename_map):
     return do_rename
 
 
-def get_lambda_code_param(params, **kwargs):
-    code = params.get('Code', {})
-    zip_file = code.get('ZipFile')
-    if zip_file and not common.is_base64(zip_file):
-        tmp_dir = common.new_tmp_dir()
-        handler_file = get_handler_file_from_name(params['Handler'], runtime=params['Runtime'])
-        tmp_file = os.path.join(tmp_dir, handler_file)
-        common.save_file(tmp_file, zip_file)
-
-        # add 'cfn-response' module to archive - see:
-        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cfn-lambda-function-code-cfnresponsemodule.html
-        cfn_response_tmp_file = get_cfn_response_mod_file()
-        cfn_response_mod_dir = os.path.join(tmp_dir, 'node_modules', 'cfn-response')
-        mkdir(cfn_response_mod_dir)
-        cp_r(cfn_response_tmp_file, os.path.join(cfn_response_mod_dir, 'index.js'))
-
-        # create zip file
-        zip_file = create_zip_file(tmp_dir, get_content=True)
-        code['ZipFile'] = zip_file
-        common.rm_rf(tmp_dir)
-    return code
-
-
-def events_put_rule_params(params, **kwargs):
-    attrs = ['ScheduleExpression', 'EventPattern', 'State', 'Description', 'Name']
-    result = select_parameters(*attrs)(params, **kwargs)
-    result['Name'] = result.get('Name') or PLACEHOLDER_RESOURCE_NAME
-
-    def wrap_in_lists(o, **kwargs):
-        if isinstance(o, dict):
-            for k, v in o.items():
-                if not isinstance(v, (dict, list)):
-                    o[k] = [v]
-        return o
-
-    pattern = result.get('EventPattern')
-    if isinstance(pattern, dict):
-        wrapped = common.recurse_object(pattern, wrap_in_lists)
-        result['EventPattern'] = json.dumps(wrapped)
-    return result
-
-
 def es_add_tags_params(params, **kwargs):
     es_arn = aws_stack.es_domain_arn(params.get('DomainName'))
     tags = params.get('Tags', [])
     return {'ARN': es_arn, 'TagList': tags}
 
 
-def lambda_permission_params(params, **kwargs):
-    result = select_parameters('FunctionName', 'Action', 'Principal')(params, **kwargs)
-    result['StatementId'] = common.short_uid()
-    return result
-
-
 def get_ddb_provisioned_throughput(params, **kwargs):
     args = params.get('ProvisionedThroughput')
     if args == PLACEHOLDER_AWS_NO_VALUE:
@@ -140,6 +91,13 @@ def get_ddb_global_sec_indexes(params, **kwargs):
     return args
 
 
+def get_ddb_kinesis_stream_specification(params, **kwargs):
+    args = params.get('KinesisStreamSpecification')
+    if args:
+        args['TableName'] = params['TableName']
+    return args
+
+
 def get_apigw_resource_params(params, **kwargs):
     result = {
         'restApiId': params.get('RestApiId'),
@@ -196,62 +154,12 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
-    'Logs::LogGroup': {
-        'create': {
-            'function': 'create_log_group',
-            'parameters': {
-                'logGroupName': 'LogGroupName'
-            }
-        },
-        'delete': {
-            'function': 'delete_log_group',
-            'parameters': {
-                'logGroupName': 'LogGroupName'
-            }
-        }
-    },
-    'Lambda::Function': {
-        'create': {
-            'function': 'create_function',
-            'parameters': {
-                'FunctionName': 'FunctionName',
-                'Runtime': 'Runtime',
-                'Role': 'Role',
-                'Handler': 'Handler',
-                'Code': get_lambda_code_param,
-                'Description': 'Description',
-                'Environment': 'Environment',
-                'Timeout': 'Timeout',
-                'MemorySize': 'MemorySize',
-                # TODO add missing fields
-            },
-            'defaults': {
-                'Role': 'test_role'
-            },
-            'types': {
-                'Timeout': int,
-                'MemorySize': int
-            }
-        },
-        'delete': {
-            'function': 'delete_function',
-            'parameters': {
-                'FunctionName': 'PhysicalResourceId'
-            }
-        }
-    },
     'Lambda::Version': {
         'create': {
             'function': 'publish_version',
             'parameters': select_parameters('FunctionName', 'CodeSha256', 'Description')
         }
     },
-    'Lambda::Permission': {
-        'create': {
-            'function': 'add_permission',
-            'parameters': lambda_permission_params
-        }
-    },
     'Lambda::EventSourceMapping': {
         'create': {
             'function': 'create_event_source_mapping',
@@ -260,7 +168,7 @@ RESOURCE_TO_FUNCTION = {
         }
     },
     'DynamoDB::Table': {
-        'create': {
+        'create': [{
             'function': 'create_table',
             'parameters': {
                 'TableName': 'TableName',
@@ -278,7 +186,10 @@ RESOURCE_TO_FUNCTION = {
                     'WriteCapacityUnits': 5
                 }
             }
-        },
+        }, {
+            'function': 'enable_kinesis_streaming_destination',
+            'parameters': get_ddb_kinesis_stream_specification
+        }],
         'delete': {
             'function': 'delete_table',
             'parameters': {
@@ -300,25 +211,6 @@ RESOURCE_TO_FUNCTION = {
             }
         }
     },
-    'Events::Rule': {
-        'create': [{
-            'function': 'put_rule',
-            'parameters': events_put_rule_params
-        }, {
-            'function': 'put_targets',
-            'parameters': {
-                'Rule': PLACEHOLDER_RESOURCE_NAME,
-                'EventBusName': 'EventBusName',
-                'Targets': 'Targets'
-            }
-        }],
-        'delete': {
-            'function': 'delete_rule',
-            'parameters': {
-                'Name': 'PhysicalResourceId'
-            }
-        }
-    },
     'IAM::Role': {
         'create': {
             'function': 'create_role',
@@ -617,6 +509,7 @@ def retrieve_resource_details(resource_id, resource_status, resources, stack_nam
 
     except DependencyNotYetSatisfied:
         return
+
     except Exception as e:
         check_not_found_exception(e, resource_type, resource, resource_status)
 
@@ -628,9 +521,11 @@ def check_not_found_exception(e, resource_type, resource, resource_status=None):
     markers = ['NoSuchBucket', 'ResourceNotFound', 'NoSuchEntity', 'NotFoundException',
         '404', 'not found', 'not exist']
     if not list(filter(lambda marker, e=e: marker in str(e), markers)):
-        LOG.warning('Unexpected error retrieving details for resource %s: %s %s - %s %s' %
-            (resource_type, e, ''.join(traceback.format_stack()), resource, resource_status))
+        LOG.warning('Unexpected error retrieving details for resource type %s: Exception: %s - %s - status: %s' %
+            (resource_type, e, resource, resource_status))
+
         return False
+
     return True
 
 
@@ -1083,7 +978,7 @@ def delete_resource(resource_id, resources, stack_name):
                 raise
 
     if res_type == 'AWS::EC2::VPC':
-        state = res['_state_']
+        state = res[KEY_RESOURCE_STATE]
         physical_resource_id = res['PhysicalResourceId'] or state.get('VpcId')
         res['PhysicalResourceId'] = physical_resource_id
 
@@ -1099,7 +994,7 @@ def delete_resource(resource_id, resources, stack_name):
                 ec2_client.delete_route_table(RouteTableId=rt['RouteTableId'])
 
     if res_type == 'AWS::EC2::Subnet':
-        state = res['_state_']
+        state = res[KEY_RESOURCE_STATE]
         physical_resource_id = res['PhysicalResourceId'] or state['SubnetId']
         res['PhysicalResourceId'] = physical_resource_id
 
@@ -1192,6 +1087,7 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     defaults = func_details.get('defaults', {})
     resource_props = resource['Properties'] = resource.get('Properties', {})
     resource_props = dict(resource_props)
+    resource_state = resource.get(KEY_RESOURCE_STATE, {})
 
     # Validate props for each resource type
     fix_resource_props_for_sdk_deployment(resource_type, resource_props)
@@ -1199,7 +1095,7 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
     if callable(params):
         params = params(resource_props, stack_name=stack_name, resources=resources, resource_id=resource_id)
     else:
-        # it could be a list ['param1', 'param2', {'apiCallParamName': 'cfResourcePropName'}]
+        # it could be a list like ['param1', 'param2', {'apiCallParamName': 'cfResourcePropName'}]
         if isinstance(params, list):
             _params = {}
             for param in params:
@@ -1222,7 +1118,8 @@ def configure_resource_via_sdk(resource_id, resources, resource_type, func_detai
                         prop_value = prop_key(resource_props, stack_name=stack_name,
                             resources=resources, resource_id=resource_id)
                     else:
-                        prop_value = resource_props.get(prop_key, resource.get(prop_key))
+                        prop_value = resource_props.get(prop_key,
+                            resource.get(prop_key, resource_state.get(prop_key)))
                     if prop_value is not None:
                         params[param_key] = prop_value
                         break
@@ -1339,6 +1236,9 @@ def run_post_create_actions(action_name, resource_id, resources, resource_type,
             if integration.get('IntegrationHttpMethod'):
                 kwargs['integrationHttpMethod'] = integration['IntegrationHttpMethod']
 
+            if integration.get('RequestTemplates'):
+                kwargs['requestTemplates'] = integration['RequestTemplates']
+
             apigateway.put_integration(
                 restApiId=api_id,
                 resourceId=res_id,
@@ -1496,7 +1396,7 @@ def determine_resource_physical_id(resource_id, resources=None, stack=None, attr
 
 def update_resource_details(stack, resource_id, details, action=None):
     resource = stack.resources.get(resource_id, {})
-    if not resource:
+    if not resource or not details:
         return
 
     resource_type = resource.get('Type') or ''
@@ -1598,6 +1498,9 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     elif res_type == 'AWS::ApiGateway::Model' and not props.get('Name'):
         props['Name'] = _generate_res_name()
 
+    elif res_type == 'AWS::ApiGateway::RequestValidator' and not props.get('Name'):
+        props['Name'] = _generate_res_name()
+
     elif res_type == 'AWS::DynamoDB::Table':
         update_dynamodb_index_resource(resource)
         props['TableName'] = props.get('TableName') or _generate_res_name()
@@ -1619,9 +1522,15 @@ def add_default_resource_props(resource, stack_name, resource_name=None,
     elif res_type == 'AWS::EC2::SecurityGroup':
         props['GroupName'] = props.get('GroupName') or _generate_res_name()
 
+    elif res_type == 'AWS::Redshift::Cluster':
+        props['ClusterIdentifier'] = props.get('ClusterIdentifier') or _generate_res_name()
+
     elif res_type == 'AWS::IAM::InstanceProfile':
         props['InstanceProfileName'] = props.get('InstanceProfileName') or _generate_res_name()
 
+    elif res_type == 'AWS::Logs::LogGroup':
+        props['LogGroupName'] = props.get('LogGroupName') or _generate_res_name()
+
     elif res_type == 'AWS::KMS::Key':
         tags = props['Tags'] = props.get('Tags', [])
         existing = [t for t in tags if t['Key'] == 'localstack-key-id']
@@ -1693,6 +1602,8 @@ class TemplateDeployer(object):
         self.apply_changes(self.stack, new_stack, stack_name=self.stack.stack_name, action='UPDATE')
 
     def delete_stack(self):
+        if not self.stack:
+            return
         self.stack.set_stack_status('DELETE_IN_PROGRESS')
         stack_resources = list(self.stack.resources.values())
         stack_name = self.stack.stack_name
