@@ -10,8 +10,8 @@ from six.moves import queue as Queue
 from six.moves.urllib.parse import urlparse
 from amazon_kclpy import kcl
 from localstack import config
-from localstack.config import HOSTNAME, USE_SSL
-from localstack.constants import LOCALSTACK_VENV_FOLDER, LOCALSTACK_ROOT_FOLDER
+from localstack.config import USE_SSL
+from localstack.constants import LOCALHOST, LOCALSTACK_VENV_FOLDER, LOCALSTACK_ROOT_FOLDER
 from localstack.utils.aws import aws_stack
 from localstack.utils.common import (
     run, TMP_THREADS, TMP_FILES, save_file, now, retry, short_uid, to_str,
@@ -135,7 +135,6 @@ class OutputReaderThread(FuncThread):
         FuncThread.__init__(self, self.start_reading, params)
         self.buffer = []
         self.params = params
-        self._stop_event = threading.Event()
         # number of lines that make up a single log entry
         self.buffer_size = 2
         # determine log level
@@ -155,10 +154,6 @@ class OutputReaderThread(FuncThread):
             self.logger.fatal = self.logger.critical
             self.logger.setLevel(self.log_level)
 
-    @property
-    def running(self):
-        return not self._stop_event.is_set()
-
     @classmethod
     def get_log_level_names(cls, min_level):
         return [logging.getLevelName(lvl) for lvl in LOG_LEVELS if lvl >= min_level]
@@ -217,9 +212,6 @@ class OutputReaderThread(FuncThread):
                     return
                 yield line.replace('\n', '')
 
-    def stop(self, quiet=True):
-        self._stop_event.set()
-
 
 class KclLogListener(object):
     def __init__(self, regex='.*'):
@@ -256,6 +248,8 @@ def get_stream_info(stream_name, log_file=None, shards=None, env=None, endpoint_
     # construct stream info
     env = aws_stack.get_environment(env)
     props_file = os.path.join(tempfile.gettempdir(), 'kclipy.%s.properties' % short_uid())
+    # make sure to convert stream ARN to stream name
+    stream_name = aws_stack.kinesis_stream_name(stream_name)
     app_name = '%s%s' % (stream_name, ddb_lease_table_suffix)
     stream_info = {
         'name': stream_name,
@@ -269,7 +263,7 @@ def get_stream_info(stream_name, log_file=None, shards=None, env=None, endpoint_
     # set local connection
     if aws_stack.is_local_env(env):
         stream_info['conn_kwargs'] = {
-            'host': HOSTNAME,
+            'host': LOCALHOST,
             'port': config.PORT_KINESIS,
             'is_secure': bool(USE_SSL)
         }
@@ -287,6 +281,8 @@ def start_kcl_client_process(stream_name, listener_script, log_file=None, env=No
         endpoint_url=None, ddb_lease_table_suffix=None, env_vars={}, region_name=None,
         kcl_log_level=DEFAULT_KCL_LOG_LEVEL, log_subscribers=[]):
     env = aws_stack.get_environment(env)
+    # make sure to convert stream ARN to stream name
+    stream_name = aws_stack.kinesis_stream_name(stream_name)
     # decide which credentials provider to use
     credentialsProvider = None
     if (('AWS_ASSUME_ROLE_ARN' in os.environ or 'AWS_ASSUME_ROLE_ARN' in env_vars) and
@@ -324,8 +320,8 @@ def start_kcl_client_process(stream_name, listener_script, log_file=None, env=No
     }
     # set parameters for local connection
     if aws_stack.is_local_env(env):
-        kwargs['kinesisEndpoint'] = '%s:%s' % (HOSTNAME, config.PORT_KINESIS)
-        kwargs['dynamodbEndpoint'] = '%s:%s' % (HOSTNAME, config.PORT_DYNAMODB)
+        kwargs['kinesisEndpoint'] = '%s:%s' % (LOCALHOST, config.PORT_KINESIS)
+        kwargs['dynamodbEndpoint'] = '%s:%s' % (LOCALHOST, config.PORT_DYNAMODB)
         kwargs['kinesisProtocol'] = get_service_protocol()
         kwargs['dynamodbProtocol'] = get_service_protocol()
         kwargs['disableCertChecking'] = 'true'
